/**
  @page LTDC, FMC_SDRAM, DMA2D, DMA, TS & e.t. functionalities use example
  
  @verbatim
  ******************* (C) COPYRIGHT 2019 Peter Maltsev       ********************
  * @file    LTDC, FMC_SDRAM, DMA2D, DMA, TC /Keyboard.h
  * @author  Peter Maltsev
  * @brief   Description of the LTDC, FMC_SDRAM, DMA2D, DMA, TS example.
  ******************************************************************************
  * февраль 2019 года
  * This is Open source software. You can place this code on your site, but don't
  * forget a link to the publication: http://pro-interes.com/category/stm32
  * Это программное обеспечение распространяется свободно. Вы можете размещать
  * его на вашем сайте, но не забудьте указать ссылку на публикацию 
  * http://pro-interes.com/category/stm32
  * Автор: Петр Мальцев / Peter Maltsev
  * mailto: t654rk@mail.ru
  * 
  ******************************************************************************
  @endverbatim 
*/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __KEYBOARD_H__
#define __KEYBOARD_H__

//------------------------------------------------------------------------------
#include "stm32f7xx_hal.h"
#include <string.h>
#include <stdlib.h>

//#include "Pf_Din_33_numerals.h"            // Пропорциональный шрифт
#include "QSPI_Addr_Table.h"
#include "Keyboard_Table_Init.h"        // Описание клавиатуры

#define LCD_COLOR_FonTTF        0xfff5f1e5  // Приятно желто-бежевый
#define LCD_COLOR_REDTTF        0xffb30216  // Красный + 15 % голубого. Какой-нибудь вишневый
#define LCD_COLOR_CYATTF        0xff00a5e7  // CYAN  полиграфический голубой
#define LCD_COLOR_BLUTTF        0xff859ec6  // приглушонный синий
#define LCD_COLOR_GrdTTF        0xff009788  // медицинский приглушонный
#define LCD_COLOR_GrlTTF        0xff00cccb  // медицинский светлый

#define LCD_ARGB1555RESET       0x00000000

#define MAX_OPERAND_STRING 16   // Разрядность определяется шириной начертания. В примере самый широкий символ "4" 
  char Operand_String[MAX_OPERAND_STRING + 3];
  uint8_t Operand_Character_Counter = MAX_OPERAND_STRING;
  char *p_Operand_String = Operand_String;
  
#define UPPER_OUTPUT_LIMIT 999999999999999.0 // Мы можем ввести с клавиатуры 16 знаков
  // Однако при переводе в double один знак точности теряется и результат будет в экспоненциальной форме
#define LOVER_OUTPUT_LIMIT 0.00000000000001
  
#define ROUNDING_FUNCTION_MULTIPLER 1000000000000000 
  // Result = round(Result * 1000000000000000) / 1000000000000000;
  // Достаточно сомнительная операция при точности double 15 знаков. Можно пользовать "trunc ()"
  // Однако отладчик показывает, что после atof (), операнд с "9" в последнем знаке БЫВАЕТ
  // представлен как ...8 и 9 в периоде.
  
#define NUMBER_KEYS_GESTURE 4
  // Колличество быстро нажатых кнопок, обрабатываемых как жест
  
  
typedef enum 
{
  no_opt = 0,
  pow_op = 1,
  div_op = 2,
  mul_op = 3,
  sub_op = 4,
  add_op = 5,
  p_c_pl = 6,   // "+ ... % ... ="   Процент плюс. По знаку "%" выводит процент. По знаку "=" прибавляет процент.
  p_c_mn = 7,   // "- ... % ... ="   Процент минус. По знаку "%" выводит процент. По знаку "=" вычитает процент.
  p_c_nds= 8    // "/ ... % ... ="   Процент НДС. По знаку "%" выводит НДС. По знаку "=" выводит сумму без НДС.
}Operation_ContentsTypeDef;

//------------------------------------------------------------------------------
// http://www.keil.com/support/man/docs/c51/c51_ap_floaterror.htm
#define NaN       0xFFFFFFFFFFFFFFFF    /* Not a number (error) */
#define plusINF   0x7FF0000000000000    /* Positive overflow    */
#define minusINF  0xFFF0000000000000    /* Negative overflow    */

union fp {
  double        fp;            /* Floating-point value */
  uint64_t      ul;           /* Unsigned long value */
};
//------------------------------------------------------------------------------

// Функция обратного вызова обработки переполнения таймера7 (время отжатия кнопки)
void TIM7_PeriodElapsedCallback(void);
void Change_DMA2D_Mode(int32_t mode, int32_t output_format, int16_t alpha);
// Функция вывода кнопки
void Rect_Move_DMA2D_32(pC_Key C_Keys, uint32_t Source_Adr, uint32_t Destination_Adr);
void Rect_Move_DMA_32(pC_Key C_Keys, uint32_t Source_Adr, uint32_t Destination_Adr);
// Функция вывода знака пропорционального шрифта
uint16_t Num_Move_DMA2D_32(pC_Key C_Keys, tImage Pf_D_Nums, uint16_t Kursor_Pos, uint32_t Destination_Adr);

// https://www.riuson.com/lcd-image-converter/examples/sources
// Функция вывода строки пропорционального шрифта
void draw_string(const char *str, pC_Key C_Keys, const tFont *font);
// Поиск символа в шрифте по коду. (Модифицирована)
const tChar *find_char_by_code(int code, const tFont *font);
// Получение следующего кода символа в строке UTF-8.  (Модифицирована)
int utf8_next_char(const char *str, int index, int *resultCode, int *nextIndex);


// Это функции для отладки touch screen
// Из событий определилось только "касание" и "отжатие"
// Причем, касание срабатывает, пока держишь кнопку, а отжатие один раз
// Хотя логично. ))))))))))
// Функция вывода строки. Из BSP добавлена Kbute_Opt
void BSP_LCD_DisplayStringAt_512(uint16_t Xpos, uint16_t Ypos, uint8_t *Text, Text_AlignModeTypdef Mode);
void BSP_LCD_DisplayChar_512(uint16_t Xpos, uint16_t Ypos, uint8_t Ascii);
void DrawChar_512(uint16_t Xpos, uint16_t Ypos, const uint8_t *c);
void BSP_LCD_DrawPixel_512(uint16_t Xpos, uint16_t Ypos, uint32_t RGB_Code);

// Найти кнопку. За время счетчика "нажимает" только первую нажатую кнопку. И ставит флаги в массиве структур всем
// остальным нажатым. Поскольку все это в цикле и нет возможности контролировать порядок "отжимает" все нажатые кнопки. 
uint8_t Rect_Find_32(pC_Key C_Keys, uint16_t x, uint16_t y, uint8_t FlagLiftUp);

// Проверить сколько кнопок нажато. В цикле проходит массив структур. Если объявить структуры отдельно, 
// то компилятор размещает их в разных частях памяти и функция работает не правильно
uint8_t Sum_Find_Flags(uint32_t Keys_Adr, const uint8_t n_Keys);  // Несколько кнопок (был жест)

// Сбросить флаги нажатых кнопок в массиве стркутур кнопок.
// Применять только для обработки жеста. После чего инициализировать калькулятор.
void Reset_Find_Flags(uint32_t Keys_Adr, const uint8_t n_Keys);    // Очистить флаги нажатых кнопок

// Преобразование к удобочитаемому (экспонециальная форма, целое число), округление и вывод результата в строку
void Convert_printf(const char *args, ...);
// Убивание пробелов, незначащих нулей, вывод на экран draw_string() и формирование из строки результата строки 
// первого операнда с возможностью редактирокания
void Result_print(double Float_Result);

// Повторяющиеся фрагменты кода
// Настройка флагов на очистку старой и ввод новой строки
void Start_input (void);
// Ввод знака
void Number_input(char number[]);
// Проверяет не в экспоненциальной ли форме результат. Если "нет" Float_Operand_1 = atof(Operand_String);
// Если "да" Float_Operand_1 = Float_Result;
void Operand1_string_conversion (void);
// Проверяет не в экспоненциальной ли форме результат. Если "нет" Float_Operand_2 = atof(Operand_String);
// Если "да" Float_Operand_2 = Float_Memory;
void Operand2_string_conversion (void);
// Получение случайного числа от нуля до единицы с 15 значащими цифрами
double random15N(void);
// Преобразование дробной части в формат целого числа с тем же знаком
double calc_746_Mod(double Result);


#endif /* __KEYBOARD_H__ */ 