/**
  @page LTDC, FMC_SDRAM, DMA2D, DMA, TC & e.t. functionalities use example
  
  @verbatim
  ******************* (C) COPYRIGHT 2019 Peter Maltsev       ********************
  * @file    LTDC, FMC_SDRAM, DMA2D, DMA, TS /Keyboard_Table_Init.h
  * @author  Peter Maltsev
  * @brief   Description of the LTDC, FMC_SDRAM, DMA2D, DMA, TC example.
  ******************************************************************************
  * февраль 2019 года
  * This is Open source software. You can place this code on your site, but don't
  * forget a link to the publication: http://pro-interes.com/category/stm32
  * Это программное обеспечение распространяется свободно. Вы можете размещать
  * его на вашем сайте, но не забудьте указать ссылку на публикацию 
  * http://pro-interes.com/category/stm32
  * Автор: Петр Мальцев / Peter Maltsev
  * mailto: t654rk@mail.ru
  * 
  ******************************************************************************
  @endverbatim 
*/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __KEYBOARD_TABLE_INIT_H
#define __KEYBOARD_TABLE_INIT_H



typedef struct
{
  int16_t Abc_X;
  int16_t Ord_Y;
  int16_t Len_X;
  int16_t Len_Y;
// В отличие от примера calculator_429I_DIS, где X и Y наоборот
// здесь абцисса действительно X, а ордината Y. 
  int16_t Press_F;              // Флаг события "Клавиша нажата"
  char    Key_Name[4];
  int16_t Reserved_Field;       // Заканчивает строку "0" и 
  // дополняет смещение адреса до 16 отдельной структуры в массиве
} C_Key, *pC_Key;

#define TAB_DIMENSION 43

// Массив структур нужен только для того, чтобы компилятор выделял под кнопки
// память одним блоком, без оптимизации
// 11,12,340,34,0, " Ind",0,    12 вместо 11 получилось из-за нечетного шрифта (просто подогнал)
C_Key   Keys_Table[TAB_DIMENSION] = {
	
		11,12,340,34,0,			" Ind",0,		362,8,110,40,0,			" Ent",0,

		8,56,52,40,0,			"  M+",0,		66,56,52,40,0,			"  MS",0,
		125,56,52,40,0,			"  MR",0,		184,56,52,40,0,			"  MC",0,
		244,56,52,40,0,			"  CE",0,		303,56,52,40,0,			"   C",0,
		362,56,52,40,0,			"BCSP",0,		420,56,52,40,0,			"   =",0,

		8,98,52,40,0,			"  xy",0,		66,98,52,40,0,			"  x2",0,
		125,98,52,40,0,			"  x3",0,		184,98,52,40,0,			"SQRT",0,
		244,98,52,40,0,			"   7",0,		303,98,52,40,0,			"   8",0,
		362,98,52,40,0,			"   9",0,		420,98,52,40,0,			"   /",0,

		8,139,52,40,0,			"  lg",0,		66,139,52,40,0,			"  ln",0,
		125,139,52,40,0,		"  ex",0,		184,139,52,40,0,		" 1/x",0,
		244,139,52,40,0,		"   4",0,		303,139,52,40,0,		"   5",0,
		362,139,52,40,0,		"   6",0,		420,139,52,40,0,		"   *",0,

		8,182,52,40,0,			"  pi",0,		66,182,52,40,0,			" sin",0,
		125,182,52,40,0,		" cos",0,		184,182,52,40,0,		" tan",0,
		244,182,52,40,0,		"   1",0,		303,182,52,40,0,		"   2",0,
		362,182,52,40,0,		"   3",0,		420,182,52,40,0,		"   -",0,

		8,224,52,40,0,			"rand",0,		66,224,52,40,0,			" div",0,
		125,224,52,40,0,		" mod",0,		184,224,52,40,0,		"   %",0,
		244,224,52,40,0,		" +/-",0,		303,224,52,40,0,		"   0",0,
		362,224,52,40,0,		"   .",0,		420,224,52,40,0,		"   +",0,
		
		0,0,480,272,0,			"FBuf",0
// FBuf  не используется как кнопка, но очень удобна для передачи всего буфера дисплея
                      };
// В принципе можно обойтись непоименованными указателями &Keys_Table[n]
// В нашем случае индивидуальная обработка функционала кнопки не добавит универсальности


pC_Key Key_ind			= &Keys_Table[0];		// indicator
pC_Key Key_entr			= &Keys_Table[1];		// Enter

pC_Key Key_ma			= &Keys_Table[2];		// add to memory
pC_Key Key_ms			= &Keys_Table[3];		// memory store
pC_Key Key_mr			= &Keys_Table[4];		// memory return/read
pC_Key Key_mc			= &Keys_Table[5];       // memory clear
pC_Key Key_ce			= &Keys_Table[6];       // clear_error
pC_Key Key_c			= &Keys_Table[7];       // clear
pC_Key Key_bs			= &Keys_Table[8];       // backspase
pC_Key Key_eq_sg		= &Keys_Table[9];		// equal sign

pC_Key Key_xy_op		= &Keys_Table[10];      // x to the power of y
pC_Key Key_x2_op		= &Keys_Table[11];      // x to the power of 2
pC_Key Key_x3_op		= &Keys_Table[12];      // x to the power of 3
pC_Key Key_sqrt_op		= &Keys_Table[13];		// square root
pC_Key Key_n_7			= &Keys_Table[14];		// 7
pC_Key Key_n_8			= &Keys_Table[15];		// 8
pC_Key Key_n_9			= &Keys_Table[16];		// 9
pC_Key Key_div_op		= &Keys_Table[17];		// divide

pC_Key Key_lg_op		= &Keys_Table[18];		// decimal logarithm
pC_Key Key_ln_op		= &Keys_Table[19];		// natural logarithm
pC_Key Key_ex_op		= &Keys_Table[20];		// exponent
pC_Key Key_inv_op		= &Keys_Table[21];      // inverse function
pC_Key Key_n_4			= &Keys_Table[22];		// 4
pC_Key Key_n_5			= &Keys_Table[23];		// 5
pC_Key Key_n_6			= &Keys_Table[24];		// 6
pC_Key Key_mul_op		= &Keys_Table[25];      // multiply operation

pC_Key Key_pi_op		= &Keys_Table[26];      // Pi
pC_Key Key_sin_op		= &Keys_Table[27];      // sinus
pC_Key Key_cos_op		= &Keys_Table[28];      // cosine
pC_Key Key_tan_op		= &Keys_Table[29];      // tangent
pC_Key Key_n_1			= &Keys_Table[30];		// 1
pC_Key Key_n_2			= &Keys_Table[31];		// 2
pC_Key Key_n_3			= &Keys_Table[32];		// 3
pC_Key Key_sub_op		= &Keys_Table[33];      // subtraction operation

pC_Key Key_rand_op		= &Keys_Table[34];		// random number sensor
pC_Key Key_idiv_op		= &Keys_Table[35];      // integer division operation
pC_Key Key_mod_op		= &Keys_Table[36];      // calculating the remainder of integer division
pC_Key Key_p_c			= &Keys_Table[37];      // percentage
pC_Key Key_ch_sg		= &Keys_Table[38];		// change the sign
pC_Key Key_n_0			= &Keys_Table[39];		// 0
pC_Key Key_d_sep		= &Keys_Table[40];		// decimal separator
pC_Key Key_add_op		= &Keys_Table[41];      // addition operation

pC_Key Key_Layer_D		= &Keys_Table[42];      // fool bufer
//------------------------------------------------------------------------------------------



#endif /* __KEYBOARD_TABLE_INIT_H */ 