/**
  @page LTDC, FMC_SDRAM, DMA2D, DMA, TS & e.t. functionalities use example
  
  @verbatim
  ******************* (C) COPYRIGHT 2019 Peter Maltsev       ********************
  * @file    LTDC, FMC_SDRAM, DMA2D, DMA, TC /Keyboard.h
  * @author  Peter Maltsev
  * @brief   Description of the LTDC, FMC_SDRAM, DMA2D, DMA, TS example.
  ******************************************************************************
  * февраль 2019 года
  * This is Open source software. You can place this code on your site, but don't
  * forget a link to the publication: http://pro-interes.com/category/stm32
  * Это программное обеспечение распространяется свободно. Вы можете размещать
  * его на вашем сайте, но не забудьте указать ссылку на публикацию 
  * http://pro-interes.com/category/stm32
  * Автор: Петр Мальцев / Peter Maltsev
  * mailto: t654rk@mail.ru
  * 
  ******************************************************************************
  @endverbatim 
*/
#ifndef __QSPI_ADDR_TABLE_H
#define __QSPI_ADDR_TABLE_H
#include <stdint.h>

typedef struct
{
    const uint8_t *data;
    uint16_t width;     // Ширина
    uint16_t height;    // Высота
    uint32_t dataSize;
} tImage;                


typedef struct
{
    long int code;
    const tImage *image;
} tChar;	        


typedef struct
{
    int length;
    const tChar *chars;
} tFont;



#define PIC_SIZE        ((uint32_t)0x88000)

//******************************************************************************
//#define IMG_NO_DATA
// Если закоментировано, то в дерево проекта нужно включить файл main_images.c.
// Предварительно лучше почистить QSPI утилитой STM32 ST-LINK utility.
// Если раскоментировать, то будут использованы адреса данных, представленные ниже.
//
// В любом случае включение main_images.c предполагает запись в QSPI,
// что делает процесс отладки "сначала скучным, потом грусным, а потом противным!"
//******************************************************************************
//------------------------------------------------------------------------------
#ifdef IMG_NO_DATA

#define TOP_ADDR        ((uint32_t)0x90000000)
#define PRESS_ADDR      ((uint32_t)0x90088000)
#define VLKSM_ADDR      ((uint32_t)0x90110000)
#define Wall_0_ADDR		((uint32_t)0x90198000)
#define Wall_1_ADDR		((uint32_t)0x90220000)
#define Wall_2_ADDR		((uint32_t)0x902A8000)

static const tImage Pf_Dn_33_0x2b = { (uint8_t*)0x903315A8, 19, 33, 8};
static const tImage Pf_Dn_33_0x2d = { (uint8_t*)0x90338AB4, 16, 33, 8};
static const tImage Pf_Dn_33_0x2e = { (uint8_t*)0x903392F4,  9, 33, 8};
static const tImage Pf_Dn_33_0x30 = { (uint8_t*)0x90331F74, 19, 33, 8};
static const tImage Pf_Dn_33_0x31 = { (uint8_t*)0x90332940, 19, 33, 8};
static const tImage Pf_Dn_33_0x32 = { (uint8_t*)0x9033816C, 18, 33, 8};
static const tImage Pf_Dn_33_0x33 = { (uint8_t*)0x9033330C, 19, 33, 8};
static const tImage Pf_Dn_33_0x34 = { (uint8_t*)0x90333CD8, 19, 33, 8};
static const tImage Pf_Dn_33_0x35 = { (uint8_t*)0x903346A4, 19, 33, 8};
static const tImage Pf_Dn_33_0x36 = { (uint8_t*)0x90335070, 19, 33, 8};
static const tImage Pf_Dn_33_0x37 = { (uint8_t*)0x90335A3C, 19, 33, 8};
static const tImage Pf_Dn_33_0x38 = { (uint8_t*)0x90336408, 19, 33, 8};
static const tImage Pf_Dn_33_0x39 = { (uint8_t*)0x90336DD4, 19, 33, 8};
static const tImage Pf_Dn_33_0x45 = { (uint8_t*)0x903377A0, 19, 33, 8};
static const tImage Pf_Dn_33_0x4f = { (uint8_t*)0x90330000, 21, 33, 8};
static const tImage Pf_Dn_33_0x52 = { (uint8_t*)0x90330AD4, 21, 33, 8};

static const tChar Pf_Dn_33_array[] = {
  {0x2b, &Pf_Dn_33_0x2b},   // character: '+'
  {0x2d, &Pf_Dn_33_0x2d},   // character: '-'
  {0x2e, &Pf_Dn_33_0x2e},   // character: '.'
  {0x30, &Pf_Dn_33_0x30},   // character: '0'
  {0x31, &Pf_Dn_33_0x31},   // character: '1'
  {0x32, &Pf_Dn_33_0x32},   // character: '2'
  {0x33, &Pf_Dn_33_0x33},   // character: '3'
  {0x34, &Pf_Dn_33_0x34},   // character: '4'
  {0x35, &Pf_Dn_33_0x35},   // character: '5'
  {0x36, &Pf_Dn_33_0x36},   // character: '6'
  {0x37, &Pf_Dn_33_0x37},   // character: '7'
  {0x38, &Pf_Dn_33_0x38},   // character: '8'
  {0x39, &Pf_Dn_33_0x39},   // character: '9'
  {0x45, &Pf_Dn_33_0x45},   // character: 'E'
  {0x4f, &Pf_Dn_33_0x4f},   // character: 'O'
  {0x52, &Pf_Dn_33_0x52},   // character: 'R'
};

const tFont Pf_Dn_33 = { 16, Pf_Dn_33_array };

#else

#define TOP_ADDR        ((uint32_t)Calc_Top_8_ARGB_512)
#define PRESS_ADDR      ((uint32_t)Calc_Press_8_ARGB_512)
#define VLKSM_ADDR      ((uint32_t)VLKSM_8_ARGB_512)
#define Wall_0_ADDR		((uint32_t)Wall_0_8_ARGB_512)
#define Wall_1_ADDR		((uint32_t)Wall_1_L_8_ARGB_512)
#define Wall_2_ADDR		((uint32_t)Mail_1_L_8_ARGB_512)

extern const unsigned char Calc_Top_8_ARGB_512[];
extern const unsigned char Calc_Press_8_ARGB_512[];
extern const unsigned char VLKSM_8_ARGB_512[];
extern const unsigned char Wall_0_8_ARGB_512[];
extern const unsigned char Wall_1_L_8_ARGB_512[];
extern const unsigned char Mail_1_L_8_ARGB_512[];

extern const uint8_t image_data_Pf_Dn_33_0x2b[];
extern const uint8_t image_data_Pf_Dn_33_0x2d[];
extern const uint8_t image_data_Pf_Dn_33_0x2e[];
extern const uint8_t image_data_Pf_Dn_33_0x30[];
extern const uint8_t image_data_Pf_Dn_33_0x31[];
extern const uint8_t image_data_Pf_Dn_33_0x32[];
extern const uint8_t image_data_Pf_Dn_33_0x33[];
extern const uint8_t image_data_Pf_Dn_33_0x34[];
extern const uint8_t image_data_Pf_Dn_33_0x35[];
extern const uint8_t image_data_Pf_Dn_33_0x36[];
extern const uint8_t image_data_Pf_Dn_33_0x37[];
extern const uint8_t image_data_Pf_Dn_33_0x38[];
extern const uint8_t image_data_Pf_Dn_33_0x39[];
extern const uint8_t image_data_Pf_Dn_33_0x45[];
extern const uint8_t image_data_Pf_Dn_33_0x4f[];
extern const uint8_t image_data_Pf_Dn_33_0x52[];

static  const tImage Pf_Dn_33_0x2b = { (uint8_t*)image_data_Pf_Dn_33_0x2b, 19, 33, 8};
static  const tImage Pf_Dn_33_0x2d = { (uint8_t*)image_data_Pf_Dn_33_0x2d, 16, 33, 8};
static  const tImage Pf_Dn_33_0x2e = { (uint8_t*)image_data_Pf_Dn_33_0x2e,  9, 33, 8};
static  const tImage Pf_Dn_33_0x30 = { (uint8_t*)image_data_Pf_Dn_33_0x30, 19, 33, 8};
static  const tImage Pf_Dn_33_0x31 = { (uint8_t*)image_data_Pf_Dn_33_0x31, 19, 33, 8};
static  const tImage Pf_Dn_33_0x32 = { (uint8_t*)image_data_Pf_Dn_33_0x32, 18, 33, 8};
static  const tImage Pf_Dn_33_0x33 = { (uint8_t*)image_data_Pf_Dn_33_0x33, 19, 33, 8};
static  const tImage Pf_Dn_33_0x34 = { (uint8_t*)image_data_Pf_Dn_33_0x34, 19, 33, 8};
static  const tImage Pf_Dn_33_0x35 = { (uint8_t*)image_data_Pf_Dn_33_0x35, 19, 33, 8};
static  const tImage Pf_Dn_33_0x36 = { (uint8_t*)image_data_Pf_Dn_33_0x36, 19, 33, 8};
static  const tImage Pf_Dn_33_0x37 = { (uint8_t*)image_data_Pf_Dn_33_0x37, 19, 33, 8};
static  const tImage Pf_Dn_33_0x38 = { (uint8_t*)image_data_Pf_Dn_33_0x38, 19, 33, 8};
static  const tImage Pf_Dn_33_0x39 = { (uint8_t*)image_data_Pf_Dn_33_0x39, 19, 33, 8};
static  const tImage Pf_Dn_33_0x45 = { (uint8_t*)image_data_Pf_Dn_33_0x45, 19, 33, 8};
static  const tImage Pf_Dn_33_0x4f = { (uint8_t*)image_data_Pf_Dn_33_0x4f, 21, 33, 8};
static  const tImage Pf_Dn_33_0x52 = { (uint8_t*)image_data_Pf_Dn_33_0x52, 21, 33, 8};

static const tChar Pf_Dn_33_array[] = {
  {0x2b, &Pf_Dn_33_0x2b},   // character: '+'
  {0x2d, &Pf_Dn_33_0x2d},   // character: '-'
  {0x2e, &Pf_Dn_33_0x2e},   // character: '.'
  {0x30, &Pf_Dn_33_0x30},   // character: '0'
  {0x31, &Pf_Dn_33_0x31},   // character: '1'
  {0x32, &Pf_Dn_33_0x32},   // character: '2'
  {0x33, &Pf_Dn_33_0x33},   // character: '3'
  {0x34, &Pf_Dn_33_0x34},   // character: '4'
  {0x35, &Pf_Dn_33_0x35},   // character: '5'
  {0x36, &Pf_Dn_33_0x36},   // character: '6'
  {0x37, &Pf_Dn_33_0x37},   // character: '7'
  {0x38, &Pf_Dn_33_0x38},   // character: '8'
  {0x39, &Pf_Dn_33_0x39},   // character: '9'
  {0x45, &Pf_Dn_33_0x45},   // character: 'E'
  {0x4f, &Pf_Dn_33_0x4f},   // character: 'O'
  {0x52, &Pf_Dn_33_0x52},   // character: 'R'
};

const tFont Pf_Dn_33 = { 16, Pf_Dn_33_array };

#endif
//------------------------------------------------------------------------------



#endif /* __QSPI_ADDR_TABLE_H */
