/**
  @page LTDC, FMC, SDRAM, QUADSPI, DMA2D, DMA, TS & e.t. functionalities use example
  
  @verbatim
  ******************* (C) COPYRIGHT 2019 Peter Maltsev       ********************
  * @file    LTDC, FMC_SDRAM, DMA2D, DMA, TC /Keyboard.c 
  * @author  Peter Maltsev
  * @brief   Description of the LTDC, FMC, SDRAM, QUADSPI, DMA2D, DMA, TS example.
  ******************************************************************************
  *  2020 
  * This is Open source software. You can place this code on your site, but don't
  * forget a link to the publication: http://pro-interes.com/category/stm32
  *     .   
  *    ,        
  * http://pro-interes.com/category/stm32
  * :   / Peter Maltsev
  * mailto: t654rk@mail.ru
  * 
  ******************************************************************************
  @endverbatim 


             
    .        
      .  Bot_Struct.xlsx.
  
              .
       ,       .
    ,    2           4.
       .
       Bot_blur_Struct.xlsx

   lcd-image-converter   .       .
         .

      QUADSPI      .

          
       
*/


/* Includes ------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h> //  -    printf   

#include "stm32746g_discovery.h"       //     
#include "lcd_bsp.h"
#include "sdram_bsp.h"
#include "qspi_bsp.h"
#include "ts_bsp.h"

#define FRB_LAYER_0  ((uint32_t)0xC0000000)
#define FRB_LAYER_1  ((uint32_t)0xC0400000)

#include "Keyboard.h"

//------------------------------------------------------------------------------

extern LTDC_HandleTypeDef hltdc;
extern LCD_DrawPropTypeDef lcdprop;
extern RNG_HandleTypeDef hrng;
extern uint32_t ActiveLayer;
extern LCD_DrawPropTypeDef DrawProp[MAX_LAYER_NUMBER];


extern __IO uint8_t EXTI_TS_Flag;

extern __IO uint8_t Kbute_Opt;  //      256 

extern DMA_HandleTypeDef hdma_memtomem_dma2_stream0;

extern DMA2D_HandleTypeDef hdma2d;

extern TIM_HandleTypeDef htim7;


uint16_t Kursor_Pos = 0;        //   .     

__IO uint8_t Line_DMA2D_flag;
__IO uint8_t Transfer_DMA2D_flag = 1;
__IO uint8_t Transfer_DMA_flag = 1;
__IO uint8_t ReloadFlag = 1;

__IO uint8_t Processing_Flag = 0;       //          
__IO uint8_t Flag_Press = 1;            //        


static TS_StateTypeDef TS_State;
uint16_t x = 0, y = 0;

Operation_ContentsTypeDef Calc_action = no_opt; // "pow", "/", "*", "-", "+", "%"


//-------------------------------------------------------------------------------
union fp F_Val_Check_Var;       //     
//-------------------------------------------------------------------------------
double Float_Operand_1 = 0;
double Float_Operand_2 = 0;
double Float_Result = 0;
double Float_Memory = 0;


static const double pi = 3.14159265358979;

uint8_t Sum_Flags_Press = 0;    //      : " "
//Attention!!!
//  Reset_Find_Flags  Key_Press_Counter = 0; Flag_Press = 1;
//           
uint8_t Key_Press_Counter = 0;  //           
int8_t Point_Flag = 0;          //   '.'
uint8_t Point_Position = 0;     //  '.'         "Backspase"
int8_t Minus_Flag = 1;          //    '-'         "Backspase"
uint8_t Minus_String_Increment = 0; //      ,     
int8_t Print_Float_Flag = 0;       //   
int8_t Print_Int_Flag = 0;         //         
int8_t EE_Fl_Flag = 0;       //       
int8_t EE_MS_Flag = 0;       //       
int8_t ERROR_Flag = 0;       //    
int8_t Rad_Flag = 0;         //    : 0 - ; 1 - 

uint8_t L1_Blending_Flag = 0;   //      (    )

uint8_t const N_Keys = TAB_DIMENSION; //    Keyboard_Table_Init.h


//------------------------------------------------------------------------------
void TIM7_PeriodElapsedCallback(void)
{
  Flag_Press = 0;
  Processing_Flag = 0;
}
//------------------------------------------------------------------------------
void HAL_LTDC_ReloadEventCallback(LTDC_HandleTypeDef *hltdc)
//    .
//  hltdc:    LTDC_HandleTypeDef,
//       LTDC.
{
  ReloadFlag = 0;
}
//------------------------------------------------------------------------------
void HAL_DMA2D_LineEventCallback(DMA2D_HandleTypeDef *hdma2d)
{
  /* DMA2D Line Transfer Complete */
  Line_DMA2D_flag = 0;
  /* Turn LED1 On */
  BSP_LED_On(LED1);
}
//------------------------------------------------------------------------------
void DMA2D_TransferComplete(DMA2D_HandleTypeDef * hdma2d)
{
  Transfer_DMA2D_flag = 0;
  /* Turn LED1 On */
  BSP_LED_On(LED1);
}
//------------------------------------------------------------------------------
void DMA2D_TransferError(DMA2D_HandleTypeDef * hdma2d)
{
  while(1)
  {
            /* Toggle LED1 with a period of 200 ms */
      BSP_LED_Toggle(LED1);
      HAL_Delay(200);
  }
}
//------------------------------------------------------------------------------
void TransferComplete(DMA_HandleTypeDef *hdma_memtomem_dma2_stream0)
{
  Transfer_DMA_flag = 0;
  /* Turn LED1 On */
  BSP_LED_On(LED1);
}
//------------------------------------------------------------------------------
void TransferError(DMA_HandleTypeDef *hdma_memtomem_dma2_stream0)
{
  while(1)
  {
          /* Toggle LED1 with a period of 200 ms */
      BSP_LED_Toggle(LED1);
      HAL_Delay(200);
  };
}

//------------------------------------------------------------------------------
void Cleaning_and_Filling_SDRAM (void)
{
  
  	/* Copy picture from QSPI to SDRAM */
        memset((void*)FRB_LAYER_1, 0, PIC_SIZE);
        memset((void*)FRB_LAYER_0, 0, PIC_SIZE);

        
	memset((void*)SDRAM_FRAME_BUF_0_ADDR_1, 0, PIC_SIZE);
        memset((void*)SDRAM_FRAME_BUF_0_ADDR_2, 0, PIC_SIZE);
        memset((void*)SDRAM_FRAME_BUF_0_ADDR_3, 0, PIC_SIZE);
  memcpy((void*)SDRAM_FRAME_BUF_0_ADDR_1, (void*)TOP_ADDR, PIC_SIZE);
  memcpy((void*)SDRAM_FRAME_BUF_0_ADDR_2, (void*)PRESS_ADDR, PIC_SIZE);
  memcpy((void*)SDRAM_FRAME_BUF_0_ADDR_3, (void*)VLKSM_ADDR, PIC_SIZE);
        memset((void*)SDRAM_FRAME_BUF_1_ADDR_1, 0, PIC_SIZE);
        memset((void*)SDRAM_FRAME_BUF_1_ADDR_2, 0, PIC_SIZE);
        memset((void*)SDRAM_FRAME_BUF_1_ADDR_3, 0, PIC_SIZE);
  memcpy((void*)SDRAM_FRAME_BUF_1_ADDR_1, (void*)Wall_0_ADDR, PIC_SIZE);
  memcpy((void*)SDRAM_FRAME_BUF_1_ADDR_2, (void*)Wall_1_ADDR, PIC_SIZE);
  memcpy((void*)SDRAM_FRAME_BUF_1_ADDR_3, (void*)Wall_2_ADDR, PIC_SIZE);
  
  memcpy((void*)FRB_LAYER_0, (void*)SDRAM_FRAME_BUF_1_ADDR_1, PIC_SIZE);
}
//------------------------------------------------------------------------------
void Change_DMA2D_Mode(int32_t mode, int32_t output_format, int16_t alpha)
{
  switch(mode)
      {
        case DMA2D_M2M_BLEND :
                              hdma2d.Init.Mode = DMA2D_M2M_BLEND;
                              hdma2d.Init.ColorMode = output_format;
                              hdma2d.Init.OutputOffset = 0;
                              hdma2d.LayerCfg[0].InputOffset = 0;
                              hdma2d.LayerCfg[0].InputColorMode = DMA2D_INPUT_ARGB8888;
                              hdma2d.LayerCfg[0].AlphaMode = DMA2D_COMBINE_ALPHA;
                              hdma2d.LayerCfg[0].InputAlpha = 255-(uint8_t)alpha;
                              hdma2d.LayerCfg[1].InputOffset = 0;
                              hdma2d.LayerCfg[1].InputColorMode = DMA2D_INPUT_ARGB8888;
                              hdma2d.LayerCfg[1].AlphaMode = DMA2D_COMBINE_ALPHA;
                              hdma2d.LayerCfg[1].InputAlpha = (uint8_t)alpha;
                              HAL_DMA2D_Init(&hdma2d);
                              HAL_DMA2D_ConfigLayer(&hdma2d, 0);
                              HAL_DMA2D_ConfigLayer(&hdma2d, 1);
                              break;
        case DMA2D_M2M_PFC :
                              hdma2d.Init.Mode = DMA2D_M2M_PFC;
                              hdma2d.Init.ColorMode = output_format;
                              hdma2d.Init.OutputOffset = 0;
                              hdma2d.LayerCfg[1].InputOffset = 0;
                              hdma2d.LayerCfg[1].InputColorMode = DMA2D_INPUT_ARGB8888;
                              hdma2d.LayerCfg[1].AlphaMode = DMA2D_NO_MODIF_ALPHA;
                              hdma2d.LayerCfg[1].InputAlpha = (uint8_t)alpha;
                              HAL_DMA2D_Init(&hdma2d);
                              HAL_DMA2D_ConfigLayer(&hdma2d, 0);
                              break;
        case DMA2D_M2M :
                              hdma2d.Init.Mode = DMA2D_M2M;
                              hdma2d.Init.ColorMode = output_format;
                              hdma2d.Init.OutputOffset = 0;
                              hdma2d.LayerCfg[1].InputOffset = 0;
                              hdma2d.LayerCfg[1].InputColorMode = DMA2D_INPUT_ARGB8888;
                              hdma2d.LayerCfg[1].AlphaMode = DMA2D_NO_MODIF_ALPHA;
                              hdma2d.LayerCfg[1].InputAlpha = (uint8_t)alpha;
                              HAL_DMA2D_Init(&hdma2d);
                              HAL_DMA2D_ConfigLayer(&hdma2d, 1);
                              break;
        case DMA2D_R2M :
                              hdma2d.Init.Mode = DMA2D_R2M;
                              hdma2d.Init.ColorMode = output_format;
                              hdma2d.Init.OutputOffset = 0;
                              HAL_DMA2D_Init(&hdma2d);
                              break;
      }
}
//------------------------------------------------------------------------------                              
void First_Frame (void)
{
HAL_Delay(1000);        //    ,   main

//       frontground
for(uint16_t i = 0; i <= 255; i = i + 3)
	{
          Change_DMA2D_Mode(DMA2D_M2M_BLEND, DMA2D_OUTPUT_ARGB1555, i);
          //    DMA2D_TransferComplete,   LineEventCallback
    
          // HAL_DMA2D_ProgramLineEvent() API enables the transfer watermark interrupt.
          // The transfer watermark interrupt is disabled once it has occurred.
          HAL_DMA2D_ProgramLineEvent (&hdma2d, 272);
          //      .  .
          
          Line_DMA2D_flag = 1;
            HAL_DMA2D_BlendingStart_IT(&hdma2d, SDRAM_FRAME_BUF_1_ADDR_2, SDRAM_FRAME_BUF_1_ADDR_1, FRB_LAYER_1, 512, 272);
                {
                  while (Line_DMA2D_flag);
                }
        }
//     frontground  
           Change_DMA2D_Mode(DMA2D_M2M_PFC, DMA2D_OUTPUT_ARGB1555, 0xFF);
           Transfer_DMA2D_flag = 1;
              HAL_DMA2D_Start_IT(&hdma2d, SDRAM_FRAME_BUF_1_ADDR_3, FRB_LAYER_1, 512, 272);
                  {
                    while (Transfer_DMA2D_flag);
                  }
//     background  
           Change_DMA2D_Mode(DMA2D_M2M, DMA2D_OUTPUT_ARGB8888, 0xFF);
           Transfer_DMA2D_flag = 1;
              HAL_DMA2D_Start_IT(&hdma2d, SDRAM_FRAME_BUF_1_ADDR_3, FRB_LAYER_0, 512, 272);
                  {
                    while (Transfer_DMA2D_flag);
                  }
//  frontground
            Change_DMA2D_Mode(DMA2D_R2M, DMA2D_OUTPUT_ARGB1555, 0xFF);                        
            Transfer_DMA2D_flag = 1;
              HAL_DMA2D_Start_IT(&hdma2d, 0x8000, FRB_LAYER_1, 512, 272);     //0b1 00000 00000 00000
                  {
                    while (Transfer_DMA2D_flag);
                  }
HAL_Delay(1500);
//      background
for(uint16_t i = 0; i <= 255; i = i + 5)
	{
          Change_DMA2D_Mode(DMA2D_M2M_BLEND, DMA2D_OUTPUT_ARGB8888, i);
          Transfer_DMA2D_flag = 1;
              HAL_DMA2D_BlendingStart_IT(&hdma2d, SDRAM_FRAME_BUF_0_ADDR_1, SDRAM_FRAME_BUF_1_ADDR_3, FRB_LAYER_0, 512, 272);
                  {
                    while (Transfer_DMA2D_flag);
                  }
        }
Change_DMA2D_Mode(DMA2D_M2M, DMA2D_OUTPUT_ARGB8888, 0xFF);
}

//  
//------------------------------------------------------------------------------
void GetPosition(uint8_t Flag_LiftUp)
{
/*  
  char Disp_String[70];
  static uint32_t int_counter1 = 0;
  static uint32_t int_counter2 = 0;
  static uint32_t int_counter3 = 0;

BSP_LCD_SetTextColor(LCD_COLOR_LIGHTYELLOW);
BSP_LCD_SetBackColor(LCD_COLOR_DARKCYAN);
BSP_LCD_SetFont(&Font20);

*/
    BSP_TS_GetState(&TS_State);
      x = TS_State.touchX[0];
      y = TS_State.touchY[0];
      
/*     
if (TS_State.touchEventId[0] == TOUCH_EVENT_CONTACT)        // 
      {
int_counter1++;
    sprintf(Disp_String, "EventId:  % 3i, % 6i ", TS_State.touchEventId[0], int_counter1);
    BSP_LCD_DisplayStringAt_512(20, 25, (uint8_t *)Disp_String, LEFT_MODE);
        SCB_CleanDCache();
      }

if (TS_State.touchEventId[0] == TOUCH_EVENT_LIFT_UP)        // 
      {
int_counter2++;        
    sprintf(Disp_String, "EventId:  % 3i, % 6i ", TS_State.touchEventId[0], int_counter2);
    BSP_LCD_DisplayStringAt_512(20, 50, (uint8_t *)Disp_String, LEFT_MODE);
        SCB_CleanDCache();
      }
if (TS_State.touchEventId[0] == TOUCH_EVENT_PRESS_DOWN)     // 
      {
int_counter3++;        
    sprintf(Disp_String, "EventId:  % 3i, % 6i ", TS_State.touchEventId[0], int_counter3);
    BSP_LCD_DisplayStringAt_512(20, 75, (uint8_t *)Disp_String, LEFT_MODE);
        SCB_CleanDCache();
      }
*/      
  
//      
  
      //--------------------- Enter
      if (Rect_Find_32(Key_entr, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_entr->Press_F == 1)&&(Processing_Flag == 0))
              Processing_Flag = 1;
          }
      //------------------------------------------------------------------------------
//--------------------- add to memory
      if (Rect_Find_32(Key_ma, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_ma->Press_F == 1)&&(Processing_Flag == 0)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
              //   -           
                  {
                    if (Calc_action == no_opt)
                    {
                      Operand1_string_conversion ();      //  Float_Operand_1;
                      Float_Memory = Float_Memory + Float_Operand_1;
                    }
                    else
                    {
                      Operand2_string_conversion ();      //  Float_Operand_2;
                      Float_Memory = Float_Memory + Float_Operand_2;
                    }
                  
                    Processing_Flag = 1;
                  }
          }
//--------------------- memory store
      if (Rect_Find_32(Key_ms, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_ms->Press_F == 1)&&(Processing_Flag == 0)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
              //   -           
                  {
                    if (Calc_action == no_opt)
                    {
                      if (EE_Fl_Flag != 0)        //   (   ) ()   
                      {
                        EE_MS_Flag = 1;
                      }
                      else
                      {
                        EE_MS_Flag = 0;
                      }
                      Operand1_string_conversion ();      //  Float_Operand_1;       
                      Float_Memory = Float_Operand_1;
                    }
                    else
                    {
                      if (EE_MS_Flag == 0)        //          
                      {
                      Operand2_string_conversion ();      //  Float_Operand_2;          
                      Float_Memory = Float_Operand_2;
                      }
                    }
                  
                    Processing_Flag = 1;
                  }
          }
//--------------------- memory return/read
      if (Rect_Find_32(Key_mr, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_mr->Press_F == 1)&&(Processing_Flag == 0))
                  {
                    Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                    Kursor_Pos = 0;
                    if (Float_Memory !=0.0)
                    {
                    Result_print (Float_Memory);
                    Kursor_Pos = 0;
                    }
                    else
                    {
                    Start_input ();
                    }
                    ERROR_Flag = 0;
                    Processing_Flag = 1;
                  }
          }
//--------------------- memory clear
      if (Rect_Find_32(Key_mc, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_mc->Press_F == 1)&&(Processing_Flag == 0))
                  {
                      Float_Memory = 0;
                      EE_MS_Flag = 0;
                      
                      Processing_Flag = 1;
                  }
          }
//--------------------- clear_error
      if (Rect_Find_32(Key_ce, x, y, Flag_LiftUp) == 1)
          {
              if ((Key_ce->Press_F == 1)&&(Processing_Flag == 0))
                    {
                      *(p_Operand_String) = 0;
                      Point_Flag = 0;
                      Point_Position = 0;
                      Minus_Flag = 1;
                      Minus_String_Increment = 0;
                      ERROR_Flag = 0;
                      EE_Fl_Flag = 0;
                      Rad_Flag = 0;
                      Calc_action = no_opt;
                      Operand_Character_Counter = MAX_OPERAND_STRING;
                      Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                      Kursor_Pos = 0;
                      if (Float_Operand_1 !=0)
                      {
                      Result_print (Float_Operand_1);     //     
                      Kursor_Pos = 0;
                      }
                                
                      Processing_Flag = 1;
                    }
          }
//--------------------- clear
      if (Rect_Find_32(Key_c, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_c->Press_F == 1)&&(Processing_Flag == 0))
                  {
                      *(p_Operand_String) = 0;
                      Point_Flag = 0;
                      Point_Position = 0;
                      Minus_Flag = 1;
                      Minus_String_Increment = 0;
                      ERROR_Flag = 0;
                      EE_Fl_Flag = 0;
                      Rad_Flag = 0;
                      Calc_action = no_opt;
                      Operand_Character_Counter = MAX_OPERAND_STRING;
                      Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                      Kursor_Pos = 0;
                      
                      Processing_Flag = 1;
                  }
          }
//--------------------- backspase
      if (Rect_Find_32(Key_bs, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_bs->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter < MAX_OPERAND_STRING)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
        // Operand_Character_Counter > 0 -  ,      
        // Operand_Character_Counter == MAX_OPERAND_STRING -   *
        // Processing_Flag = 0; -    
        // Key_bs->Press_F = 1; -  Rect_Find_32
                      {
                        if (Point_Flag == 1)
                          //        
                            {
                              if (Point_Position == (Operand_Character_Counter + 1))
                                //     
                              {
                                Point_Flag = 0;
                                Point_Position = 0;
                                //   ,   
                              }
                            }
                        *(p_Operand_String + MAX_OPERAND_STRING - ++Operand_Character_Counter + Minus_String_Increment) = 0;
                        // Minus_String_Increment = 1 -     "-",   .   Operand_Character_Counter  ,
                        //     
                        if (Point_Flag == 1)
                          //   
                            {
                               if (Point_Position == (Operand_Character_Counter + 1))
                                 //        
                               {
                                *(p_Operand_String + MAX_OPERAND_STRING - ++Operand_Character_Counter + Minus_String_Increment) = 0;
                                //      .    !
                                Point_Flag = 0;
                                Point_Position = 0;
                                // , ,       
                               }
                            }
                        Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                        //   ,    
                        draw_string(p_Operand_String, Key_ind, &Pf_Dn_33);
                        //       
                        if (Operand_Character_Counter == MAX_OPERAND_STRING)
                            // *      ++Operand_Character_Counter
                          {
                            if (Minus_Flag < 0)
                              //     ?
                            {
                              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                              //   
                              Minus_Flag = 1;
                              // Minus_Flag = 1; - "+". Minus_Flag = -1; - "-".
                              Minus_String_Increment = 0;
                              //      "-"
                              *(p_Operand_String) = 0;
                              //  
                            }
                          }
                        Kursor_Pos = 0;
                        //   )))
                        Processing_Flag = 1;
                        //  .  .     .
                            }
          }
//--------------------- equal sign
      if (Rect_Find_32(Key_eq_sg, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_eq_sg->Press_F == 1)&&(Processing_Flag == 0)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
              
              switch(Calc_action)
                          {
                          case pow_op:
                            {
                            Operand2_string_conversion ();
                            Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                                if (((Float_Operand_1 == 0) && (Float_Operand_2 <= 0)) || ((Float_Operand_1 < 0) && (fmod(Float_Operand_2, 1) !=  0)))
                                  {
                                  draw_string("ERROR", Key_ind, &Pf_Dn_33);
                                  Kursor_Pos = 0;
                                  ERROR_Flag = 1;
                                  }
                                else 
                                  {
                                F_Val_Check_Var.fp = pow(Float_Operand_1, Float_Operand_2);
                                    if ((F_Val_Check_Var.fp == NaN)  || (F_Val_Check_Var.ul == plusINF)  || (F_Val_Check_Var.ul == minusINF))
                                      {
                                      draw_string("ERROR", Key_ind, &Pf_Dn_33);
                                      Kursor_Pos = 0;
                                      ERROR_Flag = 1;
                                      }
                                    else
                                      {
                                      Float_Result = F_Val_Check_Var.fp;
                                      Result_print (Float_Result);
                                      }
                                  }
                            break;
                            }
                            
                          case div_op:
                            {
                            Operand2_string_conversion ();
                            Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                                if (Float_Operand_2 != 0)
                                {
                                Float_Result = Float_Operand_1/Float_Operand_2;
                                Result_print (Float_Result);
                                }
                                else 
                                {
                                draw_string("ERROR", Key_ind, &Pf_Dn_33);
                                Kursor_Pos = 0;
                                ERROR_Flag = 1;
                                }
                            break;
                            }
                            
                          case mul_op:
                            {
                            Operand2_string_conversion ();
                            Float_Result = Float_Operand_1 * Float_Operand_2;
                            
              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
              Result_print (Float_Result);
                            break;
                            }
                            
                          case sub_op:
                            {
                            Operand2_string_conversion ();
                            Float_Result = Float_Operand_1 - Float_Operand_2;
                            
              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
              Result_print (Float_Result);
                            break;
                            }
                            
                          case add_op:
                            {
                            Operand2_string_conversion ();
                            Float_Result = Float_Operand_1 + Float_Operand_2;
                                           
              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
              Result_print (Float_Result);
                            break;
                            }
                            
                          case p_c_pl:
                            {
                            Float_Result = Float_Operand_1 + Float_Result;
                                              
              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
              Result_print (Float_Result);
              Kursor_Pos = 0;
                            break;
                            }
              
                          case p_c_mn:
                            {
                            Float_Result = Float_Operand_1 - Float_Result;
                                              
              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
              Result_print (Float_Result);
              Kursor_Pos = 0;
                            break;
                            }
                            case p_c_nds: //  !!!   ! ))))))
                            {
                            Float_Result = Float_Operand_2 / 100 + 1;
                            Float_Result = Float_Operand_1 / Float_Result;
                                              
              Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
              Result_print (Float_Result);
              Kursor_Pos = 0;
                            break;
                            }
                          }
              
              Calc_action = no_opt;
                      
              Processing_Flag = 1;
                  }
          }
      //------------------------------------------------------------------------------
//--------------------- x to the power of y
      if (Rect_Find_32(Key_xy_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_xy_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
                //   -           
                    {
                        // Operand1_string_conversion ();
                        if (EE_Fl_Flag == 0)        //   (   ) ()    
                        {
                          if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                          {
                            Operand_String[++Operand_Character_Counter] = 0;
                            Point_Flag = 0;
                            Point_Position = 0;
                          }
                        Float_Operand_1 = atof(Operand_String);
                        }
                        else
                        {
                          Float_Operand_1 = Float_Result;   //         
                          EE_Fl_Flag = 0;
                        }
                        //______________________________
                        Calc_action = pow_op;
                
                Start_input ();
                Processing_Flag = 1;
                    }
          }
//--------------------- squaring operation
      if (Rect_Find_32(Key_x2_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_x2_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }
                    
                    Float_Result = Float_Operand_1 * Float_Operand_1;
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                   
                    Processing_Flag = 1;
                  }
          }
//--------------------- cube operation
      if (Rect_Find_32(Key_x3_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_x3_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }
                    
                    Float_Result = Float_Operand_1 * Float_Operand_1 * Float_Operand_1;
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                   
                    Processing_Flag = 1;
                  }
          }
//--------------------- square root
      if (Rect_Find_32(Key_sqrt_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_sqrt_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }
                    if (Float_Operand_1 >= 0)
                    {
                    Float_Result = sqrt(Float_Operand_1);
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                    }
                    else 
                    {

                    Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                    draw_string("ERROR", Key_ind, &Pf_Dn_33);
                    Kursor_Pos = 0;
                    ERROR_Flag = 1;
                    }
                    
                    Processing_Flag = 1;
                  }
          }
//--------------------- 7
      if (Rect_Find_32(Key_n_7, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_7->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      // Number_input("7");
                      p_Operand_String = strcat(Operand_String,"7");
                      draw_string(p_Operand_String, Key_ind, &Pf_Dn_33);
                      Kursor_Pos = 0;
                      Operand_Character_Counter --;
                      //__________________
                      Processing_Flag = 1;
                    }
          }
//--------------------- 8
      if (Rect_Find_32(Key_n_8, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_8->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("8");
                      Processing_Flag = 1;
                    }
          }
//--------------------- 9
      if (Rect_Find_32(Key_n_9, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_9->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("9");
                      Processing_Flag = 1;
                    }
          }
//--------------------- divide
      if (Rect_Find_32(Key_div_op, x, y, Flag_LiftUp) == 1)
          {
             if ((Key_div_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
                //   -           
                    {
                        // Operand1_string_conversion ();
                        if (EE_Fl_Flag == 0)        //   (   ) ()    
                        {
                          if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                          {
                            Operand_String[++Operand_Character_Counter] = 0;
                            Point_Flag = 0;
                            Point_Position = 0;
                          }
                        Float_Operand_1 = atof(Operand_String);
                        }
                        else
                        {
                          Float_Operand_1 = Float_Result;   //         
                          EE_Fl_Flag = 0;
                        }
                        //______________________________
                        Calc_action = div_op;
                
                Start_input ();
                Processing_Flag = 1;
                    }
          }
      //------------------------------------------------------------------------------
//--------------------- decimal logarithm x
      if (Rect_Find_32(Key_lg_op, x, y, Flag_LiftUp) == 1)
          {
             if ((Key_lg_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }
                    if (Float_Operand_1 > 0)
                    {
                    Float_Result = log10(Float_Operand_1);
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                    }
                    else 
                    {

                    Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                    draw_string("ERROR", Key_ind, &Pf_Dn_33);
                    Kursor_Pos = 0;
                    ERROR_Flag = 1;
                    }
                    
                    Processing_Flag = 1;
                  }
          }
//--------------------- natural logarithm x
      if (Rect_Find_32(Key_ln_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_ln_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }
                    if (Float_Operand_1 > 0)
                    {
                    Float_Result = log(Float_Operand_1);
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                    }
                    else 
                    {

                    Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                    draw_string("ERROR", Key_ind, &Pf_Dn_33);
                    Kursor_Pos = 0;
                    ERROR_Flag = 1;
                    }
                    
                    Processing_Flag = 1;
                  }
          }
//--------------------- exponential function
      if (Rect_Find_32(Key_ex_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_ex_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }

                    Float_Result = exp(Float_Operand_1);
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                    
                    Processing_Flag = 1;
                  }
          }
//--------------------- inverse function
      if (Rect_Find_32(Key_inv_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_inv_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
                //   -           
                    {
                      if (EE_Fl_Flag == 0)        //   (   ) ()    
                        {
                          if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                          {
                            Operand_String[++Operand_Character_Counter] = 0;
                            Point_Flag = 0;
                            Point_Position = 0;
                          }
                        Float_Operand_1 = atof(Operand_String);
                        }
                        else
                        {
                          Float_Operand_1 = Float_Result;   //         
                          EE_Fl_Flag = 0;
                        }
                      if (Float_Operand_1 != 0)
                        {
                        Float_Result = 1/Float_Operand_1;
                        Result_print (Float_Result);
                        Kursor_Pos = 0;
                        }
                      else 
                        {
                        Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                        draw_string("ERROR", Key_ind, &Pf_Dn_33);
                        Kursor_Pos = 0;
                        ERROR_Flag = 1;
                        }
                
                Processing_Flag = 1;
                    }
          }
//--------------------- 4
      if (Rect_Find_32(Key_n_4, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_4->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("4");
                      Processing_Flag = 1;
                    }
          }
//--------------------- 5
      if (Rect_Find_32(Key_n_5, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_5->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("5");
                      Processing_Flag = 1;
                    }
          }
//--------------------- 6
      if (Rect_Find_32(Key_n_6, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_6->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("6");
                      Processing_Flag = 1;
                    }
          }
//--------------------- multiply operation
      if (Rect_Find_32(Key_mul_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_mul_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
            //   -           
                  {
                    Operand1_string_conversion ();
                    Calc_action = mul_op;
            
                    Start_input ();
                    Processing_Flag = 1;
                  }
          }
      //------------------------------------------------------------------------------
//--------------------- Pi, rad, grad operation
      if (Rect_Find_32(Key_pi_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_pi_op->Press_F == 1)&&(Processing_Flag == 0)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
              // &(Calc_action == no_opt) , ,  pi   
                    {
                      if (Operand_Character_Counter == MAX_OPERAND_STRING)
                          {
                            Float_Result = pi;
                            Result_print (Float_Result);
                            Kursor_Pos = 0;
                            if (Calc_action != no_opt)
                              {
                               Float_Operand_2 = Float_Result;
                              }
                          }
                      else
                          {
                            if (Calc_action == no_opt)
                                  {
                                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                                        {
                                          if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                                          {
                                            Operand_String[++Operand_Character_Counter] = 0;
                                            Point_Flag = 0;
                                            Point_Position = 0;
                                          }
                                        Float_Operand_1 = atof(Operand_String);
                                        }
                                    else
                                        {
                                          Float_Operand_1 = Float_Result;   //         
                                          EE_Fl_Flag = 0;
                                        }
                                    
                                    if (Rad_Flag == 0)
                                        {
                                        Float_Result = (pi / 180) * (fmod(Float_Operand_1 + 360, 360));
                                        Result_print (Float_Result);
                                        Kursor_Pos = 0;
                                        Num_Move_DMA2D_32(Key_ind, Pf_Dn_33_0x52, Key_ind->Abc_X + 1 + Pf_Dn_33_0x52.width, FRB_LAYER_0);
                                        //    ,   "R"    + 1 
                                        Rad_Flag = 1;
                                        }
                                    else 
                                        {
                                        Float_Result = (180 / pi) * (fmod(Float_Operand_1 + 2*pi, 2*pi));
                                        Result_print (Float_Result);
                                        Kursor_Pos = 0;
                                        Rad_Flag = 0;
                                        }
                                  }
                          }

                      Processing_Flag = 1;
                    }
          }
//--------------------- sin function
      if (Rect_Find_32(Key_sin_op, x, y, Flag_LiftUp) == 1)
          {
             if ((Key_sin_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                    {
                      if (EE_Fl_Flag == 0)        //   (   ) ()    
                        {
                          if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                          {
                            Operand_String[++Operand_Character_Counter] = 0;
                            Point_Flag = 0;
                            Point_Position = 0;
                          }
                        Float_Operand_1 = atof(Operand_String);
                        }
                        else
                        {
                          Float_Operand_1 = Float_Result;   //         
                          EE_Fl_Flag = 0;
                        }
                      if (Rad_Flag == 0)
                                  {
                                   Float_Operand_1 = (pi / 180) * (fmod(Float_Operand_1 + 360, 360));
                                  }
                      Float_Result = sin(Float_Operand_1);
                      Result_print (Float_Result);
                      Kursor_Pos = 0;
                      
                      Processing_Flag = 1;
                    }
          }
//--------------------- cos function
      if (Rect_Find_32(Key_cos_op, x, y, Flag_LiftUp) == 1)
          {
             if ((Key_cos_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                    {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      }
                      else
                      {
                        Float_Operand_1 = Float_Result;   //         
                        EE_Fl_Flag = 0;
                      }
                    if (Rad_Flag == 0)
                                  {
                                   Float_Operand_1 = (pi / 180) * (fmod(Float_Operand_1 + 360, 360));
                                  }
                    Float_Result = cos(Float_Operand_1);
                    Result_print (Float_Result);
                    Kursor_Pos = 0;
                    
                    Processing_Flag = 1;
                  }                                                                                                                                             
          }
//--------------------- tan function
      if (Rect_Find_32(Key_tan_op, x, y, Flag_LiftUp) == 1)
          {
             if ((Key_tan_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                    {
                      if (EE_Fl_Flag == 0)        //   (   ) ()    
                        {
                          if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                          {
                            Operand_String[++Operand_Character_Counter] = 0;
                            Point_Flag = 0;
                            Point_Position = 0;
                          }
                        Float_Operand_1 = atof(Operand_String);
                        }
                        else
                        {
                          Float_Operand_1 = Float_Result;   //         
                          EE_Fl_Flag = 0;
                        }
                      if (Rad_Flag == 0)
                                  {
                                   Float_Operand_1 = (pi / 180) * (fmod(Float_Operand_1 + 360, 360));
                                  }
                      Float_Result = tan(Float_Operand_1);
                      Result_print (Float_Result);
                      Kursor_Pos = 0;
                      
                      Processing_Flag = 1;
                    }
          }
//--------------------- 1
      if (Rect_Find_32(Key_n_1, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_1->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("1");
                      Processing_Flag = 1;
                    }
          }
//--------------------- 2
      if (Rect_Find_32(Key_n_2, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_2->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("2");
                      Processing_Flag = 1;
                    }
          }
//--------------------- 3
      if (Rect_Find_32(Key_n_3, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_3->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("3");
                      Processing_Flag = 1;
                    }
          }
//--------------------- subtraction operation
      if (Rect_Find_32(Key_sub_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_sub_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
                //   -           
                    {
                      Operand1_string_conversion ();
                      Calc_action = sub_op;
                                
                      Start_input ();
                      Processing_Flag = 1;
                    }
          }
      //------------------------------------------------------------------------------
//--------------------- random number
      if (Rect_Find_32(Key_rand_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_rand_op->Press_F == 1)&&(Processing_Flag == 0)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
              // &(Calc_action == no_opt) , ,  pi   
                  {
                    if (Operand_Character_Counter == MAX_OPERAND_STRING)
                          {
                            Float_Result = random15N();
                            Result_print (Float_Result);
                            Kursor_Pos = 0;
                            if (Calc_action != no_opt)
                              {
                               Float_Operand_2 = Float_Result;
                              }
                          }
                    Processing_Flag = 1;
                  }
          }
//--------------------- operation of extracting the integer part of a number
      if (Rect_Find_32(Key_idiv_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_idiv_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      
                      modf(Float_Operand_1, &Float_Result);
                      Result_print (Float_Result);
                      Kursor_Pos = 0;
                      }
                                          
                    Processing_Flag = 1;
                  }
          }
//--------------------- fractional part selection operation
      if (Rect_Find_32(Key_mod_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_mod_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                  {
                    if (EE_Fl_Flag == 0)        //   (   ) ()    
                      {
                        if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
                        {
                          Operand_String[++Operand_Character_Counter] = 0;
                          Point_Flag = 0;
                          Point_Position = 0;
                        }
                      Float_Operand_1 = atof(Operand_String);
                      Float_Result = calc_746_Mod(Float_Operand_1);
                      Result_print (Float_Result);
                      Kursor_Pos = 0;
                      }
                    
                    Processing_Flag = 1;
                  }
          }
//--------------------- percentage
// https://itblog21.ru/sovety/1490-kak-schitat-protsenty-na-kalkulyatore-windows
      if (Rect_Find_32(Key_p_c, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_p_c->Press_F == 1)&&(Processing_Flag == 0)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
    //   -           
                    {
                      switch(Calc_action)
                            {
                              case div_op:  //  , Operand_2 [%].    "="   
                              {
                              Operand2_string_conversion ();        //     ( 18 %)
                              Float_Result = Float_Operand_2 / 100 + 1;     // 1,18
                              Float_Result = Float_Operand_1 / Float_Result / 100 * Float_Operand_2;
                              Calc_action = p_c_nds;
                              
                Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                Result_print (Float_Result);
                              break;
                              }
                              
                            case mul_op:    //  
                              {
                              Operand2_string_conversion ();
                              Float_Result = Float_Operand_1 / 100 * Float_Operand_2;
                              Calc_action = no_opt;
                              
                Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                Result_print (Float_Result);
                              break;
                              }
                              
                            case sub_op:    //    "="  
                              {
                              Operand2_string_conversion ();
                              Float_Result = Float_Operand_1 / 100 * Float_Operand_2;
                              Calc_action = p_c_mn;
                              
                Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                Result_print (Float_Result);
                              break;
                              }
                              
                            case add_op:    //    "="  
                              {
                              Operand2_string_conversion ();
                              Float_Result = Float_Operand_1 / 100 * Float_Operand_2;
                              Calc_action = p_c_pl;
                              
                Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                Result_print (Float_Result);
                              break;
                              }
                            }
                      
                Processing_Flag = 1;
                    }
          }
//--------------------- change the sign
      if (Rect_Find_32(Key_ch_sg, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_ch_sg->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter + 1 > 0)&&(Operand_Character_Counter != MAX_OPERAND_STRING)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
          //   "-"
          //   Operand_Character_Counter + 1 > 0,     
          //    "0"   "."   ,    Operand_Character_Counter == MAX_OPERAND_STRING
                    {
                    Minus_Flag = Minus_Flag * (-1);
                    //   
                    if (Minus_Flag < 0)
                    {
                      for(int i = MAX_OPERAND_STRING - Operand_Character_Counter; i >= 0; i--)
                        //   
                      {
                        *(p_Operand_String + i + 1) = *(p_Operand_String + i);
                      }
                      *p_Operand_String = '-'; 
                      //    "-"
                      Minus_String_Increment = 1;
                      // ,    
                    }
                    else
                    {
                      for(int i = 0; i <= MAX_OPERAND_STRING - Operand_Character_Counter + Minus_String_Increment; i++)
                        //   
                      {
                        *(p_Operand_String + i) = *(p_Operand_String + i + 1);
                      }
                      Minus_String_Increment = 0;
                      //    
                      Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
                      //   .  -.  .
                    }
                draw_string(p_Operand_String, Key_ind, &Pf_Dn_33);
                //       
                Kursor_Pos = 0;
                //      draw_string...    Reserved
                Processing_Flag = 1;
                //  .     
                    }
          }
//--------------------- 0
      if (Rect_Find_32(Key_n_0, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_n_0->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      Number_input("0");
                      Processing_Flag = 1;
                    }
          }
//--------------------- decimal separator
      if (Rect_Find_32(Key_d_sep, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_d_sep->Press_F == 1)&&(Processing_Flag == 0)&&(Operand_Character_Counter > 0)&&(Point_Flag == 0)&&(EE_Fl_Flag == 0)&&(ERROR_Flag == 0))
                    {
                      p_Operand_String = strcat(Operand_String,".");
                      draw_string(p_Operand_String, Key_ind, &Pf_Dn_33);
                      Point_Flag = 1;
                      Point_Position = Operand_Character_Counter;
                      Kursor_Pos = 0;
                      Operand_Character_Counter --;
                      
                      Processing_Flag = 1;
                    }
          }
//--------------------- addition operation
      if (Rect_Find_32(Key_add_op, x, y, Flag_LiftUp) == 1)
          {
            if ((Key_add_op->Press_F == 1)&&(Processing_Flag == 0)&&(Calc_action == no_opt)&&((Point_Flag <= (MAX_OPERAND_STRING - Operand_Character_Counter))||(EE_Fl_Flag != 0))&&(ERROR_Flag == 0))
                //   -           
                    {
                        Operand1_string_conversion ();
                        Calc_action = add_op;
                
                        Start_input ();   
                        Processing_Flag = 1;
                    }
          }
//------------------------------------------------------------------------------
//--------------------- Gesture processing
      //  "".    STMPE811    .
      //    .     .    , 
      //             
      if ((Sum_Find_Flags((uint32_t)&Keys_Table[0], N_Keys)) >= NUMBER_KEYS_GESTURE)
         {
           Rect_Move_DMA2D_32(Key_Layer_D, TOP_ADDR, FRB_LAYER_0);
           Reset_Find_Flags((uint32_t)&Keys_Table[0], N_Keys);
                
       switch (L1_Blending_Flag)
       {
       case 0:      //  ""
           {
            //       frontground
            for(uint16_t i = 0; i < 255; i = i + 3)
                    {
                      Change_DMA2D_Mode(DMA2D_M2M_BLEND, DMA2D_OUTPUT_ARGB1555, i);
                      Transfer_DMA2D_flag = 1;
                        HAL_DMA2D_BlendingStart_IT(&hdma2d, SDRAM_FRAME_BUF_0_ADDR_3, SDRAM_FRAME_BUF_0_ADDR_1, FRB_LAYER_1, 512, 272);
                            {
                              while (Transfer_DMA2D_flag);
                            }
                    }
            Change_DMA2D_Mode(DMA2D_M2M, DMA2D_OUTPUT_ARGB8888, 0xFF);
            L1_Blending_Flag = 1;
           break;
           }
         case 1:       //  ""
           {
             for(uint16_t i = 0; i < 255; i = i + 3)
                    {
                      Change_DMA2D_Mode(DMA2D_M2M_BLEND, DMA2D_OUTPUT_ARGB1555, i);
                      Transfer_DMA2D_flag = 1;
                        HAL_DMA2D_BlendingStart_IT(&hdma2d, SDRAM_FRAME_BUF_0_ADDR_1, SDRAM_FRAME_BUF_0_ADDR_3, FRB_LAYER_1, 512, 272);
                            {
                              while (Transfer_DMA2D_flag);
                            }
                    }
             Change_DMA2D_Mode(DMA2D_M2M, DMA2D_OUTPUT_ARGB8888, 0xFF);
             Transfer_DMA2D_flag = 1;
                        HAL_DMA2D_Start_IT(&hdma2d, SDRAM_FRAME_BUF_0_ADDR_1, FRB_LAYER_0, 512, 272);
                            {
                              while (Transfer_DMA2D_flag);
                            }
             Change_DMA2D_Mode(DMA2D_R2M, DMA2D_OUTPUT_ARGB1555, 0xFF);
             Transfer_DMA2D_flag = 1;
                        HAL_DMA2D_Start_IT(&hdma2d, 0x8000, FRB_LAYER_1, 512, 272);     //0b1 00000 00000 00000
                            {
                              while (Transfer_DMA2D_flag);
                            }
            Change_DMA2D_Mode(DMA2D_M2M, DMA2D_OUTPUT_ARGB8888, 0xFF);
            L1_Blending_Flag = 0;
           break;
           }
        }
       //   -    ,      
       //         
       //        'C' ()
          *(p_Operand_String) = 0;
          Point_Flag = 0;
          Point_Position = 0;
          Minus_Flag = 1;
          Minus_String_Increment = 0;
          Rad_Flag = 0;
          ERROR_Flag = 0;
          EE_Fl_Flag = 0;
          Float_Operand_1 = 0;
          Float_Operand_2 = 0;
          Float_Result = 0;
          Float_Memory = 0;
          Calc_action = no_opt;
          Operand_Character_Counter = MAX_OPERAND_STRING;
          Key_Press_Counter = 0;
          Kursor_Pos = 0;
      //    Flag_Press = 1;   //    Update        !!!
      //    HAL_TIM_Base_Stop_IT(&htim7);    // Disable the TIM Update interrupt
          __HAL_TIM_DISABLE_IT(&htim7, TIM_IT_UPDATE);

          Processing_Flag = 0;        //   HAL_TIM_Base_Stop_IT()
          Flag_Press = 1;             //   HAL_TIM_Base_Stop_IT()
      //
      //    ,        
      // touch screen   
      //           .
      //       ,     !
      //    GOTO.
          }
//------------------------------------------------------------------------------
if (Flag_LiftUp == 1) EXTI_TS_Flag = 0; //    ,  
// ,        .   .
BSP_TS_ResetTouchData(&TS_State);
//------------------------------------------------------------------------------
}

//  .    ""    .       
//  .           ""   . 
//------------------------------------------------------------------------------
uint8_t Rect_Find_32(pC_Key C_Keys, uint16_t x, uint16_t y, uint8_t FlagLiftUp)
{

uint8_t Find_flag = 0;

if ((C_Keys->Press_F == 0) && ( x > C_Keys->Abc_X ) && ( x < C_Keys->Abc_X +  C_Keys->Len_X) && ( y > C_Keys->Ord_Y ) && ( y < C_Keys->Ord_Y +  C_Keys->Len_Y))
  {                             //   
//Attention!!!
//  Reset_Find_Flags  Key_Press_Counter = 0; Flag_Press = 1;
//           
        C_Keys->Press_F = 1;            // ,    

       if ((Flag_Press != 0) && (Key_Press_Counter++  == 0))   //   .  
       {
//        Rect_Move_DMA2D_32(C_Keys, (uint32_t)&Calk_Press_ARGB, FRB_LAYER_0);    // 
        Rect_Move_DMA2D_32(C_Keys, PRESS_ADDR, FRB_LAYER_0);    // 
//        Rect_Move_DMA_32(C_Keys, PRESS_ADDR, FRB_LAYER_0);
//______________________________________________________________________________
            // __HAL_TIM_SET_COUNTER(&htim7, htim7.Init.Period); //   
      __HAL_TIM_SET_COUNTER(&htim7, 0);
                  // ((__HANDLE__)->Instance->CNT = (__COUNTER__))
      
            // __HAL_TIM_DISABLE_IT(&htim7, TIM_IT_UPDATE);
        HAL_TIM_Base_Start_IT(&htim7);  // Enable the TIM Update interrupt, Enable the Peripheral
            // __HAL_TIM_ENABLE_IT(&htim7, TIM_IT_UPDATE);
                  // ((__HANDLE__)->Instance->DIER |= (__INTERRUPT__))
                  // TIM_DIER_UIE           (0x1U << TIM_DIER_UIE_Pos)
            //__HAL_TIM_ENABLE(&htim7);
                  // ((__HANDLE__)->Instance->CR1|=(TIM_CR1_CEN))
                  // TIM_CR1_CEN            (0x1U << TIM_CR1_CEN_Pos)

            // TIMn->ARR   = NNN;           // 
                    // TIMx_ARR    ,
                    //    0  TIMx_ARR,    
                    //   .
            // TIMn->CR1  |= TIM_CR1_DIR;   //  ,   
            // TIMn->CR1  |= TIM_CR1_ARPE;  //     
            // TIMn->EGR  |= TIM_EGR_UG;    //  
            // TIMn->DIER |= TIM_DIER_UIE;  //    
            // TIMn->BDTR |= TIM_BDTR_MOE;  //     
            // TIMn->CR1  |= TIM_CR1_CEN;   //  
//______________________________________________________________________________

//  HAL_TIM_Base_Start_IT(&htim7);   
//    ,   Flag_Press = 0;
//     .
        Flag_Press = 1;
        Find_flag = 1;          //  
       }
       else
       {        //  ...     )))
        Find_flag = 2;          //   
       }
   }
else 
   {                             //    .      ...
     if ((C_Keys->Press_F == 1) && ((Flag_Press == 0) || (FlagLiftUp == 1)))      //       
         {  //      .     "" 
//          Rect_Move_DMA2D_32(C_Keys, (uint32_t)&Calk_Top_ARGB, FRB_LAYER_0);
          Rect_Move_DMA2D_32(C_Keys, TOP_ADDR, FRB_LAYER_0);
          C_Keys->Press_F = 0;              // ,    
          Find_flag = 0;          //   
          if (--Key_Press_Counter == 0)     //    
              {
//                 HAL_TIM_Base_Stop_IT(&htim7);    // Disable the TIM Update interrupt
               __HAL_TIM_DISABLE_IT(&htim7, TIM_IT_UPDATE);    
                 //   HAL_TIM_Base_Stop_IT().  Update        !!!
                  Flag_Press = 1;        //       
                 if (FlagLiftUp == 1)
                 {
                   Processing_Flag = 0;
                 }
              }
         }
   }
  return Find_flag;
}

//   ()    .     .    , 
//             
//------------------------------------------------------------------------------
uint8_t Sum_Find_Flags(uint32_t Keys_Adr, const uint8_t n_Keys)
{
uint8_t i;
Sum_Flags_Press = 0;   //    Key_Press_Counter (    )
for (i = 1; i <= n_Keys; i++ )
{
  if (*(__IO uint16_t*) (Keys_Adr + 16 * i - 8) > 0)
  {
    Sum_Flags_Press++;  //     
  }                     //        
}
  return Sum_Flags_Press;
}

//    .
//------------------------------------------------------------------------------
void Reset_Find_Flags(uint32_t Keys_Adr, const uint8_t n_Keys)
{
uint8_t i;

Sum_Flags_Press = 0;

for (i = 1; i <= n_Keys; i++ )
    {
      *(__IO uint16_t*) (Keys_Adr + 16 * i - 8) = 0;
    }
}
//------------------------------------------------------------------------------
//       1
void Operand1_string_conversion (void)
{
if (EE_Fl_Flag == 0)        //   (   ) ()    
            {
              if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
              {
                Operand_String[++Operand_Character_Counter] = 0;
                Point_Flag = 0;
                Point_Position = 0;
              }
            Float_Operand_1 = atof(Operand_String);
            }
            else
            {
              Float_Operand_1 = Float_Result;   //         
              EE_Fl_Flag = 0;
            }
}
//------------------------------------------------------------------------------
//       2 (      )
void Operand2_string_conversion (void)
{
if (EE_Fl_Flag == 0)        //   (   ) ()    
            {
              if ((Point_Position - Operand_Character_Counter) == 1) //      ,   
              {
                Operand_String[++Operand_Character_Counter] = 0;
                Point_Flag = 0;
                Point_Position = 0;
              }
            Float_Operand_2 = atof(Operand_String);
            }
            else
            {
              Float_Operand_2 = Float_Memory;   //         
              EE_Fl_Flag = 0;
            }
}

//  
//------------------------------------------------------------------------------
void Number_input(char number[])
{
  
  p_Operand_String = strcat(Operand_String, number);
    draw_string(p_Operand_String, Key_ind, &Pf_Dn_33);
    Kursor_Pos = 0;
    Operand_Character_Counter --;
}
//------------------------------------------------------------------------------
//             
void Start_input (void)
{
Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);
    Kursor_Pos = 0;
    *(p_Operand_String) = 0;
    Point_Flag = 0;
    Point_Position = 0;
    Minus_Flag = 1;
    Minus_String_Increment = 0;
    Operand_Character_Counter = MAX_OPERAND_STRING;
}
//------------------------------------------------------------------------------
//    ( ,  ),      
void Result_print (double Result)
{
int64_t Integer_Result = 0;

    Rect_Move_DMA2D_32(Key_ind, TOP_ADDR, FRB_LAYER_0);

    if ((Result > UPPER_OUTPUT_LIMIT) || (Result < -UPPER_OUTPUT_LIMIT) || ((Result < LOVER_OUTPUT_LIMIT) && (Result > 0)) || ((Result > -LOVER_OUTPUT_LIMIT) && (Result < 0)))
    {
      if (Result < 0.0)
      {
        EE_Fl_Flag =1;
        Convert_printf("%13.9E", Result);
      }
           else
           {
             EE_Fl_Flag =1;
             Convert_printf("%13.10E", Result);
           }
    }
    else 
    {
      if (fmod(Result,1) == 0.0)
        {
          Integer_Result = (int64_t)Result;
          EE_Fl_Flag = 0;
          Convert_printf("%16lld", Integer_Result);
        }
      else
      {
        Print_Float_Flag = 1;                  //   
        EE_Fl_Flag = 0;
        Result = round(Result * ROUNDING_FUNCTION_MULTIPLER) / ROUNDING_FUNCTION_MULTIPLER;
        // sqrt(2) = 1,4142135623730950488016887242097
        // DBL_MAX 1.7976931348623158e+308 Maximum value
        //    15-    ...
        //                     1.414213562373095144 * 1000000000000000 = 1414213562373095.144;
        //                     round(1414213562373095.144) = 1414213562373095;
        //                     1414213562373095 / 1000000000000000 = 1.414213562373095;
        //  9.12345678999999.  atof 9.1234567899999898
        //   .  9.1234567899999898   9.12345678999999
        //  ,    4-5       ...8(9)
        //   .999999999999999 (   ,     )
        //  .     ,    .
        Convert_printf("%16.15f", Result);
      }
    }
}
   
//------------------------------------------------------------------------------
//  ,  ,    draw_string()       
//     
void Convert_printf(const char *args, ...)
// https://www.opennet.ru/man.shtml?topic=vsnprintf&category=3&russian=0
{
  char Result_String[100];
  char *p_Result_String = Result_String;
  
  va_list ap;   // va_list -    ,   
                //   .  ap -   argument pointer
  va_start(ap, args);   //  va_start  ap ,  
                        //     .     
                        //  ,    ap.      
                        //   ;     
                        //     
  char len = vsnprintf(Result_String, sizeof(Result_String), args, ap); // sizeof -     Result_String
  // int vsnprintf( char *s, size_t n, const char *format, va_list ap );
  //           .
  //          va list
  va_end(ap);


// -1.41421356237309'\0'
//  1.41421356237309'\0'
  if (Result_String[0] == '-')
      {
        Result_String[MAX_OPERAND_STRING + 1] = 0;    //   ,   
      }
  else
      {
        Result_String[MAX_OPERAND_STRING] = 0;    
      }
  
  
// -12.33 x 98.76 = -1218.6984
// ' ',   ' ',   ' ',   '-',   '1',   '2',   '1',  '8',   '.',   '6',   '9',   '8',   '4',   ' ',   ' ',   '\0'
  
  if (Print_Float_Flag == 1)   //    (!)
    {
      Print_Float_Flag = 0;
      for (int i = strlen(Result_String) - 1; i > 0; i--)
        {
           if(Result_String[i] == '0')
           {
               Result_String[i] = 0;
           }
           else if(Result_String[i] == '.')
           { 
                Result_String[i] = 0;
                break;
           }
            else  break;
        }
    }
// ' ',   ' ',   ' ',   '-',   '1',   '2',   '1',   '8',   '.',   '6',   '9',   '8',   '4',  '\0'  
      int j = strspn(Result_String, " "); //         
      for(int i = 0; i <= (int)(strlen(Result_String) - j); i++)
                  //   
                {
                  *(p_Result_String + i) = *(p_Result_String + i + j);
                }
// '-',   '1',   '2',   '1',   '8',   '.',   '6',   '9',   '8',   '4',    '\0'
//-------------------------------------------  
      draw_string(Result_String, Key_ind, &Pf_Dn_33);           //  .
          Kursor_Pos = 0;
      
  if (EE_Fl_Flag == 0)        //        
      {
      //-------------------------------------------
        strcpy(Operand_String, Result_String);               //      .
          Operand_Character_Counter = MAX_OPERAND_STRING - strlen(Operand_String);
      // '-',   '1',   '2',   '1',   '8',   '.',   '6',   '9',   '8',   '4',  '\0'
      //  6      7      8      9      10     11     12     13     14     15
      // Operand_Character_Counter == 6
          
            Point_Flag = 0;
            Point_Position = 0;
            if (strchr(Operand_String, '.') != NULL)
            {
              Point_Position = MAX_OPERAND_STRING - (uint32_t)(strchr(Operand_String, '.') - p_Operand_String);
      // '-',   '1',   '2',   '1',   '8',   '.',   '6',   '9',   '8',   '4',  '\0'
      //  6      7      8      9      10     11     12     13     14     15
      // Point_Position == 11
              
              Point_Flag = 1;
            }
            
            Minus_Flag = 1;
            Minus_String_Increment = 0;
            if (Operand_String[0] == '-')     //     Result_String
            {
              Minus_Flag = -1;
              Minus_String_Increment = 1;
              ++Operand_Character_Counter;
              ++Point_Position;
      //     .   Minus_String_Increment == 1
      //             backspase
      // Operand_Character_Counter = 7, Point_Position = 12, Minus_String_Increment = 1
            }
      }
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// https://www.riuson.com/lcd-image-converter/examples/sources
//   lcd-image-converter  STM   LTDC
//      ""   ,     
//           )))))
//

//      .
const tChar *find_char_by_code(int code, const tFont *font)
{
    int count = font->length;
    int first = 0;
    int last = count - 1;
    int mid = 0;

    if (count > 0)
    {
        if ((code >= font->chars[0].code) && (code <= font->chars[count - 1].code))
        {
            while (last >= first)
            {
                mid = first + ((last - first) / 2);

                if (font->chars[mid].code < code)
                    first = mid + 1;
                else
                    if (font->chars[mid].code > code)
                        last = mid - 1;
                    else
                        break;
            }

            if (font->chars[mid].code == code)
                return (&font->chars[mid]);
        }
    }

    return (0);
}

//             DMA2D
//       
// . .     -,   -.
//------------------------------------------------------------------------------
//       UTF-8.
//         ( 1-).
int utf8_next_char(const char *str, int index, int *resultCode, int *nextIndex)
{
    *resultCode = 0;
    unsigned char c;
    unsigned int code = 0;
    int result = 0;
    int skip = 0;
    *resultCode = 0;
    *nextIndex = -1;
        while (index >= 0)
        {
            c = *(str + index);
            index--;

            // msb
            if (skip == 0)
            {
                // if range 0x00010000-0x001fffff
                if ((c & 0xf8) == 0xf0)
                {
                    skip = 3;
                    code = c;
                }
                // if range 0x00000800-0x0000ffff
                else if ((c & 0xf0) == 0xe0)
                {
                    skip = 2;
                    code = c;
                }
                // if range 0x00000080-0x000007ff
                else if ((c & 0xe0) == 0xc0)
                {
                    skip = 1;
                    code = c;
                }
                // if range 0x00-0x7f
                else //if ((c & 0x80) == 0x00)
                {
                    skip = 0;
                    code = c;
                }
            }
            else // not msb
            {
                code = code << 8;
                code |= c;
                skip--;
            }
            if (skip == 0)
            {
                // completed
                *resultCode = code;
                *nextIndex = index;
                result = 1;
                break;
            }
        }
        return (result);
}

//------------------------------------------------------------------------------
//  .
void draw_string(const char *str, pC_Key C_Keys, const tFont *font)
{
    int len = strlen(str);
    int index = len - 1;
    int code = 0;
    int nextIndex;
    Kursor_Pos = C_Keys->Abc_X + C_Keys->Len_X;
    while (index >= 0)
    {
        if (utf8_next_char(str, index, &code, &nextIndex) != 0)
        {
            const tChar *ch = find_char_by_code(code, font);
            if (ch != 0)
            {
			Kursor_Pos = Num_Move_DMA2D_32(Key_ind, *(ch->image), Kursor_Pos, FRB_LAYER_0);
            }
        }
        index = nextIndex;
        if (nextIndex < 0)
            break;
    }
}

//  
/*
int utf8_next_char(const char *str, int start, int *resultCode, int *nextIndex)
{
    int len = 0;
    int index = 0;
    *resultCode = 0;

    while (*(str + index) != 0)
    {
        len++;
        index++;
    }

    unsigned char c;
    unsigned int code = 0;
    int result = 0;
    int skip = 0;

    *resultCode = 0;
    *nextIndex = -1;

    if (start >= 0 && start < len)
    {
        index = start;

        while (index < len)
        {
            c = *(str + index);
            index++;

            // msb
            if (skip == 0)
            {
                // if range 0x00010000-0x001fffff
                if ((c & 0xf8) == 0xf0)
                {
                    skip = 3;
                    code = c;
                }
                // if range 0x00000800-0x0000ffff
                else if ((c & 0xf0) == 0xe0)
                {
                    skip = 2;
                    code = c;
                }
                // if range 0x00000080-0x000007ff
                else if ((c & 0xe0) == 0xc0)
                {
                    skip = 1;
                    code = c;
                }
                // if range 0x00-0x7f
                else //if ((c & 0x80) == 0x00)
                {
                    skip = 0;
                    code = c;
                }
            }
            else // not msb
            {
                code = code << 8;
                code |= c;
                skip--;
            }
            if (skip == 0)
            {
                // completed
                *resultCode = code;
                *nextIndex = index;
                result = 1;
                break;
            }
        }
    }
    return (result);
}

void draw_string(const char *str, int x, int y, const tFont *font)
{
    int len = ustrlen(str);
    int index = 0;
    int code = 0;
    int x1 = x;
    int nextIndex;
    while (index < len)
    {
        if (utf8_next_char(str, index, &code, &nextIndex) != 0)
        {
            const tChar *ch = find_char_by_code(code, font);
            if (ch != 0)
            {
                draw_bitmap_mono(x1, y, ch->image);
                x1 += ch->image->width;
            }
        }
        index = nextIndex;
        if (nextIndex < 0)
            break;
    }
}
*/
//         15  
//------------------------------------------------------------------------------
double random15N(void)
{
  char n1[21] = "                     ";
  char n2[11] = "           ";
  char *p_n1 = n1;
  char *p_n2 = n2;
  uint32_t random_01 = 0;
  uint32_t random_02 = 0;
  int8_t i,j;
    HAL_RNG_GenerateRandomNumber(&hrng, &random_01);
            sprintf(n1,"%10u", random_01);
            // ' ',   ' ',   ' ',   '-',   '1',   '2',   '1',   '8',   '.',   '6',   '9',   '8',   '4',  '\0'  
            j = strspn(n1, " "); //         
            for(i = 0; i <= (int8_t)(strlen(n1) - j); i++)
            //   
                {
                  *(p_n1 + i) = *(p_n1 + i + j);
                }
            // '-',   '1',   '2',   '1',   '8',   '.',   '6',   '9',   '8',   '4',  '\0'
                            
    HAL_RNG_GenerateRandomNumber(&hrng, &random_02);
            sprintf(n2,"%10u", random_02);
            j = strspn(n2, " ");
            for(i = 0; i <= (int8_t)(strlen(n2) - j); i++)
            //   
                {
                  *(p_n2 + i) = *(p_n2 + i + j);
                }
                            
    strcat (n1, n2);
                            
            *(p_n1 + (strlen(n1)-16)) = '.';
            *(p_n1 + (strlen(n1)-17)) = '0';
            
            for(i = 0; i < (int8_t)(strlen(n1)-17); i++)
                  {
                     *(p_n1 + i) = ' ';
                  }
            
            j = strspn(n1, " ");
            for(i = 0; i <= (int8_t)(strlen(n1) - j); i++)
            //   
                {
                  *(p_n1 + i) = *(p_n1 + i + j);
                }
    return atof(n1);
}
//           
//------------------------------------------------------------------------------
double calc_746_Mod(double Result)
{
  char Result_String[40];
  char *p_Result_String = Result_String;
  int8_t i,j;
  int8_t minus_found = 0;
  
  if (fmod(Result,1) == 0.0)
        {
          return Result;
        }
  else
        {
          Result = round(Result * ROUNDING_FUNCTION_MULTIPLER) / ROUNDING_FUNCTION_MULTIPLER;
          
          sprintf(Result_String,"%16.15f", Result);
        }
	  

  if (Result_String[0] == '-')
        {
          minus_found = 1;
          Result_String[MAX_OPERAND_STRING + 1] = 0;    //   ,   
        }
  else
        {
          Result_String[MAX_OPERAND_STRING] = 0;    
        }
  
  
// -12.33 x 98.76 = -1218.6984
// ' ',   ' ',   ' ',   '-',   '1',   '2',   '1',  '8',   '.',   '6',   '9',   '8',   '4',   ' ',   ' ',   '\0'
 
  for (i = strlen(Result_String) - 1; i > 0; i--)
        {
           if(Result_String[i] == '0')
           {
               Result_String[i] = 0;
           }
           else if(Result_String[i] == '.')
           { 
                Result_String[i] = 0;
                break;
           }
            else  break;
        }
//    ,       fmod(Result,1) != 0.0
  for (i = 0; i < strlen(Result_String); i++)
        {
           if(Result_String[i] == '.')
           { 
                 if (minus_found == 1)
                 {
                   Result_String[i] = 0x2d;    // "-"
                 }
                 else
                 {
                   Result_String[i] = 0x20;    // " "
                 }
             break;
           }
            else
           {
                Result_String[i] = 0x20;
           }
        }
  j = strspn(Result_String, " "); //         
  for(i = 0; i <= (int8_t)(strlen(Result_String) - j); i++)
                  //   
           {
                *(p_Result_String + i) = *(p_Result_String + i + j);
           }
  return atof(Result_String);
}
//------------------------------------------------------------------------------
//   () DMA2D
// ARGB 
uint16_t Num_Move_DMA2D_32(pC_Key C_Keys, tImage Pf_D_Nums, uint16_t Kursor_Pos, uint32_t Destination_Adr)
{

//    ...   
/*
uint32_t S_Init_Mode = hdma2d.Init.Mode;                        //   
uint32_t S_LayerCfg = hdma2d.LayerCfg[1].InputColorMode;        //   DMA2D
uint32_t S_InputOffset = hdma2d.LayerCfg[1].InputOffset;
uint32_t S_OutputOffset = hdma2d.Init.OutputOffset;

uint32_t addr_source_offset = 0;
uint32_t addr_destination_offset = 0;


                      hdma2d.Init.Mode = DMA2D_M2M;
                      
                      //  ,    CM [3: 0]  DMA2D_FGPFCCR,
                      //         .
                      // if(LayerIdx == 0)
                      //   {
                      // Write DMA2D BGPFCCR register
                      // MODIFY_REG(hdma2d->Instance->BGPFCCR,..
                      // else
                      //   {
                      // Write DMA2D FGPFCCR register
                      // MODIFY_REG(hdma2d->Instance->FGPFCCR,..
                      //  :
                      hdma2d.LayerCfg[1].InputColorMode = DMA2D_INPUT_ARGB8888;
                      
                      //  DMA2D = 0
                      hdma2d.LayerCfg[1].InputOffset = 0;
                      //     ,     
                      //  DMA2D  =    * 
                      hdma2d.Init.OutputOffset = BSP_LCD_GetXSize() + Kbute_Opt - Pf_D_Nums.width;
                      
                      HAL_DMA2D_Init(&hdma2d);
                      HAL_DMA2D_ConfigLayer(&hdma2d, 1);
                      

addr_source_offset = (uint32_t)(Pf_D_Nums.data);
addr_destination_offset = Destination_Adr + 4 * ((C_Keys->Ord_Y) * (BSP_LCD_GetXSize() + Kbute_Opt) + Kursor_Pos - Pf_D_Nums.width);

    Transfer_DMA2D_flag = 1;
    HAL_DMA2D_Start_IT(&hdma2d, addr_source_offset, addr_destination_offset, Pf_D_Nums.width, Pf_D_Nums.height);
while (Transfer_DMA2D_flag);
    {
    }
hdma2d.Init.Mode = S_Init_Mode;                        // 
hdma2d.LayerCfg[1].InputColorMode = S_LayerCfg;        //   DMA2D
hdma2d.LayerCfg[1].InputOffset = S_InputOffset;
hdma2d.Init.OutputOffset = S_OutputOffset;
HAL_DMA2D_Init(&hdma2d);
HAL_DMA2D_ConfigLayer(&hdma2d, 1);
  
  
  return Kursor_Pos - Pf_D_Nums.width;
}

*/

// stm32f429xx.h -  .
uint32_t S_CR = DMA2D->CR;              //   
uint32_t S_FGPFCCR = DMA2D->FGPFCCR;    //   DMA2D
uint32_t S_FGOR = DMA2D->FGOR;
uint32_t S_OOR = DMA2D->OOR;

// -
//      DMA2D     .
//  FIFO     ,   
//    ,   DMA2D_FGMAR,    ,
//  DMA2D_OMAR.
//  ,    CM [3: 0]  DMA2D_FGPFCCR,
//         .
//      DMA2D_NLR  DMA2D_FGOR
//  ,    DMA2D_NLR  DMA2D_OOR  .

DMA2D->CR &= ~DMA2D_CR_MODE_Msk;                // hdma2d.Init.Mode = DMA2D_M2M;

DMA2D->FGMAR = ((uint32_t)(Pf_D_Nums.data));     // Source_Adr
DMA2D->OMAR = Destination_Adr + 4 * ((C_Keys->Ord_Y) * (BSP_LCD_GetXSize() + Kbute_Opt) + Kursor_Pos - Pf_D_Nums.width);

DMA2D->FGOR = 0;                                                // hdma2d.LayerCfg[N].InputOffset
//  DMA2D = 0;
//     ,     
DMA2D->OOR = BSP_LCD_GetXSize() + Kbute_Opt - Pf_D_Nums.width; // hdma2d.Init.OutputOffset
//  DMA2D  =    * 
//hdma2d.Init.OutputOffset = Lcd_Width + Kbute_Opt - len_x;

DMA2D->FGPFCCR  &= ~DMA2D_BGPFCCR_CM_Msk;                // Input color mode CM[3:0]

//    :
//hdma2d.Init.ColorMode = DMA2D_OUTPUT_RGB565;
//HAL_DMA2D_Init(&hdma2d);
//---------------------------------------
//DMA2D->OPFCCR  &= ~DMA2D_OPFCCR_CM_Msk;                 // DMA2D_OUTPUT_ARGB8888;
//      .)))

DMA2D->NLR = 0; //   -    ,
//     ,  !
DMA2D->NLR |= (Pf_D_Nums.height << DMA2D_NLR_NL_Pos) | (Pf_D_Nums.width << DMA2D_NLR_PL_Pos); // DMA2D_NLR_NL_Pos (0U)
//DMA2D_NLR_NL - Number of Lines
//DMA2D_NLR_PL - Pixel per Lines
// *       
//lcd-image-converter     ,     

    Transfer_DMA2D_flag = 1;
    
DMA2D->CR |= DMA2D_CR_TCIE_Msk;         //      
DMA2D->CR |= DMA2D_CR_START_Msk;         //  DMA2D
//HAL_DMA2D_Start_IT(&hdma2d, addr_source_offset, addr_destination_offset, len_x, len_y);


while (Transfer_DMA2D_flag);
    {
    }
    
DMA2D->CR = S_CR;              // 
DMA2D->FGPFCCR = S_FGPFCCR;    //   DMA2D
DMA2D->FGOR = S_FGOR;
DMA2D->OOR = S_OOR;


  return Kursor_Pos - Pf_D_Nums.width;
}

//------------------------------------------------------------------------------
//  DMA. DataLength: The length of data to be transferred from source to destination
// ARGB                len_x + 1
//                    DMA_PDATAALIGN_WORD
void Rect_Move_DMA_32(pC_Key C_Keys, uint32_t Source_Adr, uint32_t Destination_Adr)
{
//        DMA    
uint32_t Save_PeriphDataAlignment = hdma_memtomem_dma2_stream0.Init.PeriphDataAlignment;
uint32_t Save_MemDataAlignment = hdma_memtomem_dma2_stream0.Init.MemDataAlignment;
//    ,    Color_Mode,   

uint32_t Lcd_Width;
uint32_t ypos;
int16_t ord_y, abc_x, len_y, len_x;

ord_y = C_Keys->Ord_Y;
abc_x = C_Keys->Abc_X;
len_y = C_Keys->Len_Y;
len_x = C_Keys->Len_X;

Lcd_Width = BSP_LCD_GetXSize(); //   BSP

                      hdma_memtomem_dma2_stream0.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;
                      hdma_memtomem_dma2_stream0.Init.MemDataAlignment = DMA_MDATAALIGN_WORD;
                      HAL_DMA_Init(&hdma_memtomem_dma2_stream0);
                      
              for(ypos = ord_y; ypos <= (ord_y + len_y); ypos++)
                 {
                  Transfer_DMA_flag = 1;
  HAL_DMA_Start_IT(&hdma_memtomem_dma2_stream0, ((uint32_t) Source_Adr) + (4 * (ypos * (Lcd_Width + Kbute_Opt) + abc_x)), ((uint32_t) Destination_Adr) + (4 * (ypos * (Lcd_Width + Kbute_Opt) + abc_x)), len_x + 1);
                  while(Transfer_DMA_flag)
                        {
                        }
                 }
              
                      hdma_memtomem_dma2_stream0.Init.PeriphDataAlignment = Save_PeriphDataAlignment;
                      hdma_memtomem_dma2_stream0.Init.MemDataAlignment = Save_MemDataAlignment;
                      HAL_DMA_Init(&hdma_memtomem_dma2_stream0);
}

  

//------------------------------------------------------------------------------
//  DMA2D
// ARGB 
void Rect_Move_DMA2D_32(pC_Key C_Keys, uint32_t Source_Adr, uint32_t Destination_Adr)
{

uint32_t S_Init_Mode = hdma2d.Init.Mode;                        //   
uint32_t S_LayerCfg = hdma2d.LayerCfg[1].InputColorMode;        //   DMA2D
uint32_t S_InputOffset = hdma2d.LayerCfg[1].InputOffset;
uint32_t S_OutputOffset = hdma2d.Init.OutputOffset;

uint32_t Lcd_Width;
int16_t ord_y, abc_x, len_y, len_x;

uint32_t addr_source_offset = 0;
uint32_t addr_destination_offset = 0;

ord_y = C_Keys->Ord_Y;
abc_x = C_Keys->Abc_X;
len_y = C_Keys->Len_Y;
len_x = C_Keys->Len_X;

Lcd_Width = BSP_LCD_GetXSize(); //   BSP



                      hdma2d.Init.Mode = DMA2D_M2M;
                      
                      //  ,    CM [3: 0]  DMA2D_FGPFCCR,
                      //         .
                      // if(LayerIdx == 0)
                      //   {
                      // Write DMA2D BGPFCCR register
                      // MODIFY_REG(hdma2d->Instance->BGPFCCR,..
                      // else
                      //   {
                      // Write DMA2D FGPFCCR register
                      // MODIFY_REG(hdma2d->Instance->FGPFCCR,..
                      //  :
                      hdma2d.LayerCfg[1].InputColorMode = DMA2D_INPUT_ARGB8888;
                      
                      //  DMA2D =     
                      hdma2d.LayerCfg[1].InputOffset = Lcd_Width + Kbute_Opt - len_x;
                      //  DMA2D    .       HAL_DMA2D_ConfigLayer
                      hdma2d.Init.OutputOffset = hdma2d.LayerCfg[1].InputOffset;
                      
                      HAL_DMA2D_Init(&hdma2d);
                      HAL_DMA2D_ConfigLayer(&hdma2d, 1);
                      

addr_source_offset = Source_Adr + 4 * (ord_y * (Lcd_Width + Kbute_Opt) + abc_x);
addr_destination_offset = Destination_Adr + 4 * (ord_y * (Lcd_Width + Kbute_Opt) + abc_x);

            Transfer_DMA2D_flag = 1;
            SCB_CleanDCache();
    HAL_DMA2D_Start_IT(&hdma2d, addr_source_offset, addr_destination_offset, len_x, len_y);
            while (Transfer_DMA2D_flag);
                {
                }
hdma2d.Init.Mode = S_Init_Mode;                        // 
hdma2d.LayerCfg[1].InputColorMode = S_LayerCfg;        //   DMA2D
hdma2d.LayerCfg[1].InputOffset = S_InputOffset;
hdma2d.Init.OutputOffset = S_OutputOffset;
HAL_DMA2D_Init(&hdma2d);
HAL_DMA2D_ConfigLayer(&hdma2d, 1);
}
/*
//    ...   

uint32_t S_CR = DMA2D->CR;              //   
uint32_t S_FGPFCCR = DMA2D->FGPFCCR;    //   DMA2D
uint32_t S_FGOR = DMA2D->FGOR;
uint32_t S_OOR = DMA2D->OOR;

uint32_t Lcd_Width;                     //      1 
int16_t ord_y, abc_x, len_y, len_x;

ord_y = C_Keys->Ord_Y;
abc_x = C_Keys->Abc_X;
len_y = C_Keys->Len_Y;
len_x = C_Keys->Len_X;

Lcd_Width = BSP_LCD_GetXSize(); //   BSP

// -
//      DMA2D     .
//  FIFO     ,   
//    ,   DMA2D_FGMAR,    ,
//  DMA2D_OMAR.
//  ,    CM [3: 0]  DMA2D_FGPFCCR,
//         .
//      DMA2D_NLR  DMA2D_FGOR
//  ,    DMA2D_NLR  DMA2D_OOR  .

DMA2D->CR &= ~DMA2D_CR_MODE_Msk;                // hdma2d.Init.Mode = DMA2D_M2M;

DMA2D->FGMAR = Source_Adr + 4 * (ord_y * (Lcd_Width + Kbute_Opt) + abc_x);
DMA2D->OMAR = Destination_Adr + 4 * (ord_y * (Lcd_Width + Kbute_Opt) + abc_x);

DMA2D->FGOR = Lcd_Width + Kbute_Opt - len_x;            // hdma2d.LayerCfg[N].InputOffset
DMA2D->OOR = Lcd_Width + Kbute_Opt - len_x;             // hdma2d.Init.OutputOffset

DMA2D->FGPFCCR  &= ~DMA2D_BGPFCCR_CM_Msk;                // Input color mode CM[3:0]

//    :
//hdma2d.Init.ColorMode = DMA2D_OUTPUT_RGB565;
//HAL_DMA2D_Init(&hdma2d);
//---------------------------------------
//DMA2D->OPFCCR  &= ~DMA2D_OPFCCR_CM_Msk;                 // DMA2D_OUTPUT_ARGB8888;
//      .)))

DMA2D->NLR = 0; //   -    ,
//     ,  !
DMA2D->NLR |= (len_y << DMA2D_NLR_NL_Pos) | (len_x << DMA2D_NLR_PL_Pos); // DMA2D_NLR_NL_Pos (0U)

    Transfer_DMA2D_flag = 1;
    
DMA2D->CR |= DMA2D_CR_TCIE_Msk;         //      
DMA2D->CR |= DMA2D_CR_START_Msk;         //  DMA2D
//HAL_DMA2D_Start_IT(&hdma2d, addr_source_offset, addr_destination_offset, len_x, len_y);


while (Transfer_DMA2D_flag);
    {
    }
DMA2D->CR = S_CR;              // 
DMA2D->FGPFCCR = S_FGPFCCR;    //   DMA2D
DMA2D->FGOR = S_FGOR;
DMA2D->OOR = S_OOR;
}
*/
//------------------------------------------------------------------------------
//     touch screen
//     ""  ""
// ,  ,   ,    
//  . ))))))))))
//------------------------------------------------------------------------------
/**
  * @brief  Displays characters on the LCD.
  * @param  Xpos: X position (in pixel)
  * @param  Ypos: Y position (in pixel)   
  * @param  Text: Pointer to string to display on LCD
  * @param  Mode: Display mode
  *          This parameter can be one of the following values:
  *            @arg  CENTER_MODE
  *            @arg  RIGHT_MODE
  *            @arg  LEFT_MODE   
  * @retval None
  */
//  xsize = ((BSP_LCD_GetXSize() + Kbute_Opt)....
//  BSP_LCD_DisplayChar_512
void BSP_LCD_DisplayStringAt_512(uint16_t Xpos, uint16_t Ypos, uint8_t *Text, Text_AlignModeTypdef Mode)
{

  uint16_t ref_column = 1, i = 0;
  uint32_t size = 0, xsize = 0; 
  uint8_t  *ptr = Text;
  
  
  /* Get the text size */
  while (*ptr++) size ++ ;
  
  /* Characters number per line */
  xsize = ((BSP_LCD_GetXSize() + Kbute_Opt)/DrawProp[ActiveLayer].pFont->Width);
  
  switch (Mode)
  {
  case CENTER_MODE:
    {
      ref_column = Xpos + ((xsize - size)* DrawProp[ActiveLayer].pFont->Width) / 2;
      break;
    }
  case LEFT_MODE:
    {
      ref_column = Xpos;
      break;
    }
  case RIGHT_MODE:
    {
      ref_column = - Xpos + ((xsize - size)*DrawProp[ActiveLayer].pFont->Width);
      break;
    }    
  default:
    {
      ref_column = Xpos;
      break;
    }
  }
  
  /* Check that the Start column is located in the screen */
  if ((ref_column < 1) || (ref_column >= 0x8000))
  {
    ref_column = 1;
  }

  /* Send the string character by character on LCD */
  while ((*Text != 0) & (((BSP_LCD_GetXSize() + Kbute_Opt - (i*DrawProp[ActiveLayer].pFont->Width)) & 0xFFFF) >= DrawProp[ActiveLayer].pFont->Width))
  {
    /* Display one character on LCD */
    BSP_LCD_DisplayChar_512(ref_column, Ypos, *Text);
    /* Decrement the column position by 16 */
    ref_column += DrawProp[ActiveLayer].pFont->Width;
    /* Point on the next character */
    Text++;
    i++;
  }  
}

/**
  * @brief  Displays one character.
  * @param  Xpos: Start column address
  * @param  Ypos: Line where to display the character shape.
  * @param  Ascii: Character ascii code
  *           This parameter must be a number between Min_Data = 0x20 and Max_Data = 0x7E 
  * @retval None
  */
//  BSP_LCD_DrawPixel_512
//  DrawChar_512
void BSP_LCD_DisplayChar_512(uint16_t Xpos, uint16_t Ypos, uint8_t Ascii)
{
  DrawChar_512(Xpos, Ypos, &DrawProp[ActiveLayer].pFont->table[(Ascii-' ') *\
    DrawProp[ActiveLayer].pFont->Height * ((DrawProp[ActiveLayer].pFont->Width + 7) / 8)]);
}

void DrawChar_512(uint16_t Xpos, uint16_t Ypos, const uint8_t *c)
{
  uint32_t i = 0, j = 0;
  uint16_t height, width;
  uint8_t  offset;
  uint8_t  *pchar;
  uint32_t line;
  
  height = DrawProp[ActiveLayer].pFont->Height;
  width  = DrawProp[ActiveLayer].pFont->Width;
  
  offset =  8 *((width + 7)/8) -  width ;
  
  for(i = 0; i < height; i++)
  {
    pchar = ((uint8_t *)c + (width + 7)/8 * i);
    
    switch(((width + 7)/8))
    {
      
    case 1:
      line =  pchar[0];      
      break;
      
    case 2:
      line =  (pchar[0]<< 8) | pchar[1];      
      break;
      
    case 3:
    default:
      line =  (pchar[0]<< 16) | (pchar[1]<< 8) | pchar[2];      
      break;
    } 
    
    for (j = 0; j < width; j++)
    {
      if(line & (1 << (width- j + offset- 1))) 
      {
        BSP_LCD_DrawPixel_512((Xpos + j), Ypos, DrawProp[ActiveLayer].TextColor);
      }
      else
      {
        BSP_LCD_DrawPixel_512((Xpos + j), Ypos, DrawProp[ActiveLayer].BackColor);
      } 
    }
    Ypos++;
  }
}

/**
  * @brief  Draws a pixel on LCD.
  * @param  Xpos: X position
  * @param  Ypos: Y position
  * @param  RGB_Code: Pixel color in ARGB mode (8-8-8-8)
  * @retval None
  */
//  + Kbute_Opt)....
void BSP_LCD_DrawPixel_512(uint16_t Xpos, uint16_t Ypos, uint32_t RGB_Code)
{
  /* Write data value to all SDRAM memory */
  if(hltdc.LayerCfg[ActiveLayer].PixelFormat == LTDC_PIXEL_FORMAT_RGB565)
  { /* RGB565 format */
    *(__IO uint16_t*) (hltdc.LayerCfg[ActiveLayer].FBStartAdress + (2*(Ypos*(BSP_LCD_GetXSize() + Kbute_Opt) + Xpos))) = (uint16_t)RGB_Code;
  }
  else
  { /* ARGB8888 format */
    *(__IO uint32_t*) (hltdc.LayerCfg[ActiveLayer].FBStartAdress + (4*(Ypos*(BSP_LCD_GetXSize() + Kbute_Opt) + Xpos))) = RGB_Code;
  }
}
 
















